# -*- coding: utf-8 -*-
from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer
from plone.app.contenttypes.tests.robot.variables import TEST_FOLDER_ID
from plone.app.event.testing import PAEvent_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.testing import z2
from zope.interface import alsoProvides

import pkg_resources


def set_browserlayer(request):
    """Set the BrowserLayer for the request.

    We have to set the browserlayer manually, since importing the profile alone
    doesn't do it in tests.
    """
    alsoProvides(request, IPloneAppContenttypesLayer)


class PloneAppContenttypes(PloneSandboxLayer):

    defaultBases = (PAEvent_FIXTURE, PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.app.contenttypes
        self.loadZCML(package=plone.app.contenttypes)
        import plone.app.event.dx
        self.loadZCML(package=plone.app.event.dx)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'plone.app.contenttypes:default')
        portal.portal_workflow.setDefaultChain('simple_publication_workflow')

    def tearDownPloneSite(self, portal):
        applyProfile(portal, 'plone.app.contenttypes:uninstall')


class PloneAppContenttypesRobot(PloneAppContenttypes):
    """Same as the default but with a added folder 'robot-test-folder'.
    """

    defaultBases = (
        PAEvent_FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE)

    def setUpPloneSite(self, portal):
        portal.acl_users.userFolderAddUser(
            SITE_OWNER_NAME, SITE_OWNER_PASSWORD, ['Manager'], [])
        login(portal, SITE_OWNER_NAME)
        super(PloneAppContenttypesRobot, self).setUpPloneSite(portal)
        portal.invokeFactory('Folder', id=TEST_FOLDER_ID, title=u'Test Folder')

    def tearDownPloneSite(self, portal):
        login(portal, 'admin')
        portal.manage_delObjects([TEST_FOLDER_ID])
        super(PloneAppContenttypesRobot, self).tearDownPloneSite(portal)


class PloneAppContenttypesMigration(PloneSandboxLayer):
    """ A setup that installs the old default AT-Types to migrate them to
    Dexterity. The profile of pac is not only in the individual tests.
    """

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):

        # prepare installing Products.ATContentTypes
        import Products.ATContentTypes
        self.loadZCML(package=Products.ATContentTypes)
        z2.installProduct(app, 'Products.Archetypes')
        z2.installProduct(app, 'Products.ATContentTypes')
        z2.installProduct(app, 'plone.app.blob')

        # prepare installing plone.app.collection
        try:
            pkg_resources.get_distribution('plone.app.collection')
            z2.installProduct(app, 'plone.app.collection')
        except pkg_resources.DistributionNotFound:
            pass

        # prepare installing plone.app.contenttypes
        z2.installProduct(app, 'Products.DateRecurringIndex')

        import plone.app.contenttypes
        self.loadZCML(package=plone.app.contenttypes)
        import plone.app.referenceablebehavior
        self.loadZCML(package=plone.app.referenceablebehavior)

    def setUpPloneSite(self, portal):
        # install Products.ATContentTypes manually if profile is available
        # (this is only needed for Plone >= 5)
        profiles = [x['id'] for x in portal.portal_setup.listProfileInfo()]
        if 'Products.ATContentTypes:default' in profiles:
            applyProfile(portal, 'Products.ATContentTypes:default')

        # enable old Topic
        portal.portal_types.Topic.global_allow = True

        # install plone.app.collections manually if profile is available
        # (this is only needed for Plone >= 5)
        if 'plone.app.collection:default' in profiles:
            applyProfile(portal, 'plone.app.collection:default')

        applyProfile(portal, 'plone.app.referenceablebehavior:default')

    def tearDownPloneSite(self, portal):
        applyProfile(portal, 'plone.app.contenttypes:uninstall')

    def tearDownZope(self, app):
        try:
            pkg_resources.get_distribution('plone.app.collection')
            z2.uninstallProduct(app, 'plone.app.collection')
        except pkg_resources.DistributionNotFound:
            pass
        z2.uninstallProduct(app, 'plone.app.blob')
        z2.uninstallProduct(app, 'Products.ATContentTypes')
        z2.uninstallProduct(app, 'Products.Archetypes')


PLONE_APP_CONTENTTYPES_FIXTURE = PloneAppContenttypes()
PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE,),
    name='PloneAppContenttypes:Integration'
)
PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE,),
    name='PloneAppContenttypes:Functional'
)
PLONE_APP_CONTENTTYPES_ROBOT_FIXTURE = PloneAppContenttypesRobot()
PLONE_APP_CONTENTTYPES_ROBOT_TESTING = FunctionalTesting(
    bases=(
        PLONE_APP_CONTENTTYPES_ROBOT_FIXTURE,
        z2.ZSERVER_FIXTURE
    ),
    name='PloneAppContenttypes:Robot'
)
PLONE_APP_CONTENTTYPES_MIGRATION_FIXTURE = PloneAppContenttypesMigration()
PLONE_APP_CONTENTTYPES_MIGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENTTYPES_MIGRATION_FIXTURE,),
    name='PloneAppContenttypes:Migration'
)
PLONE_APP_CONTENTTYPES_MIGRATION_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENTTYPES_MIGRATION_FIXTURE,),
    name='PloneAppContenttypes:Migration_Functional'
)



# -*- coding: utf-8 -*-
from plone.app.contenttypes import _
from plone.app.z3cform.widget import QueryStringFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.browser.syndication.adapters import CollectionFeed as BaseCollectionFeed  # noqa
from Products.CMFPlone.interfaces.syndication import IFeed
from Products.CMFPlone.interfaces.syndication import ISyndicatable
from zope import schema
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import implementer
from zope.interface import provider
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


@implementer(IVocabularyFactory)
class MetaDataFieldsVocabulary(object):

    def __call__(self, context):
        cat = getToolByName(context, 'portal_catalog')
        items = [
            SimpleTerm(column, column, column)
            for column in cat.schema()
        ]
        return SimpleVocabulary(items)


MetaDataFieldsVocabularyFactory = MetaDataFieldsVocabulary()


@provider(IFormFieldProvider, ISyndicatable)
class ICollection(model.Schema):

    query = schema.List(
        title=_(u'Search terms'),
        description=_(u'Define the search terms for the items you want '
                      u'to list by choosing what to match on. '
                      u'The list of results will be dynamically updated'),
        value_type=schema.Dict(value_type=schema.Field(),
                               key_type=schema.TextLine()),
        required=False,
        missing_value=''
    )
    form.widget('query', QueryStringFieldWidget)

    sort_on = schema.TextLine(
        title=_(u'label_sort_on', default=u'Sort on'),
        description=_(u'Sort the collection on this index'),
        required=False,
    )

    sort_reversed = schema.Bool(
        title=_(u'label_sort_reversed', default=u'Reversed order'),
        description=_(u'Sort the results in reversed order'),
        required=False,
    )

    limit = schema.Int(
        title=_(u'Limit'),
        description=_(u'Limit Search Results'),
        required=False,
        default=1000,
        min=1,
    )

    item_count = schema.Int(
        title=_(u'label_item_count', default=u'Item count'),
        description=_(u'Number of items that will show up in one batch.'),
        required=False,
        default=30,
        min=1,
    )

    customViewFields = schema.List(
        title=_(u'Table Columns'),
        description=_(u"Select which fields to display when "
                      u"'Tabular view' is selected in the display menu."),
        default=['Title', 'Creator', 'Type', 'ModificationDate'],
        value_type=schema.Choice(
            vocabulary='plone.app.contenttypes.metadatafields'),
        required=False,
    )


class ISyndicatableCollection(ISyndicatable):
    """Marker interface for syndicatable collections.
    """


@implementer(ICollection)
@adapter(IDexterityContent)
class Collection(object):

    def __init__(self, context):
        self.context = context

    def results(self, batch=True, b_start=0, b_size=None,
                sort_on=None, limit=None, brains=False,
                custom_query=None):
        if custom_query is None:
            custom_query = {}
        querybuilder = getMultiAdapter((self.context, self.context.REQUEST),
                                       name='querybuilderresults')
        sort_order = 'reverse' if self.sort_reversed else 'ascending'
        if not b_size:
            b_size = self.item_count
        if not sort_on:
            sort_on = self.sort_on
        if not limit:
            limit = self.limit
        return querybuilder(
            query=self.query, batch=batch, b_start=b_start, b_size=b_size,
            sort_on=sort_on, sort_order=sort_order,
            limit=limit, brains=brains, custom_query=custom_query
        )

    def selectedViewFields(self):
        """Returns a list of all metadata fields from the catalog that were
           selected.

        The template expects a tuple/list of (id, title) of the field.

        """
        _mapping = {}
        vocab = getUtility(IVocabularyFactory,
                           name='plone.app.contenttypes.metadatafields')
        for field in vocab(self.context):
            _mapping[field.value] = (field.value, field.title)
        ret = [_mapping[field] for field in self.customViewFields]
        return ret

    # Getters and setters for our fields.

    def _set_sort_reversed(self, value):
        self.context.sort_reversed = value

    def _get_sort_reversed(self):
        return getattr(self.context, 'sort_reversed', None)

    sort_reversed = property(_get_sort_reversed, _set_sort_reversed)

    def _set_item_count(self, value):
        self.context.item_count = value

    def _get_item_count(self):
        return getattr(self.context, 'item_count', 30)

    item_count = property(_get_item_count, _set_item_count)

    def _set_sort_on(self, value):
        self.context.sort_on = value

    def _get_sort_on(self):
        return getattr(self.context, 'sort_on', None)

    sort_on = property(_get_sort_on, _set_sort_on)

    def _set_limit(self, value):
        self.context.limit = value

    def _get_limit(self):
        return getattr(self.context, 'limit', 1000)

    limit = property(_get_limit, _set_limit)

    def _set_query(self, value):
        self.context.query = value

    def _get_query(self):
        return getattr(self.context, 'query', None)

    query = property(_get_query, _set_query)

    def _set_customViewFields(self, value):
        self.context.customViewFields = value

    def _get_customViewFields(self):
        # Note: in corner cases customViewFields might be None, but we
        # always want a list.
        return getattr(self.context, 'customViewFields', []) or []

    customViewFields = property(_get_customViewFields, _set_customViewFields)


@implementer(IFeed)
class CollectionFeed(BaseCollectionFeed):

    def _brains(self):
        return ICollection(self.context).results(batch=False)[:self.limit]



# -*- coding: utf-8 -*-
from plone.app.contenttypes.interfaces import IEvent
from plone.app.contenttypes.migration.field_migrators import datetime_fixer
from plone.dexterity.interfaces import IDexterityContent
from plone.dexterity.interfaces import IDexterityFTI
from plone.event.utils import default_timezone
from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2Base
from Products.CMFCore.utils import getToolByName
from Products.contentmigration.basemigrator.migrator import CMFItemMigrator
from Products.contentmigration.basemigrator.walker import CatalogWalker
from zExceptions import NotFound
from zope.annotation.interfaces import IAnnotations
from zope.component import queryUtility
from zope.component.hooks import getSite

import importlib
import logging


logger = logging.getLogger(__name__)


def migrate(portal, migrator):
    """return a CatalogWalker instance in order
    to have its output after migration"""
    walker = CatalogWalker(portal, migrator)()
    return walker


class ContentMigrator(CMFItemMigrator):
    """Base for contentish DX
    """

    def migrate_atctmetadata(self):
        self.new.exclude_from_nav = self.old.exclude_from_nav


def migrate_to_pa_event(context):
    # Install plone.app.event
    context.runAllImportStepsFromProfile('profile-plone.app.event:default')
    # Re-import types to get newest Event type
    context.runImportStepFromProfile(
        'profile-plone.app.contenttypes:default',
        'typeinfo'
    )
    portal = getSite()
    migrate(portal, DXOldEventMigrator)


class DXOldEventMigrator(ContentMigrator):
    """Migrator for 1.0 plone.app.contenttypes DX events"""

    src_portal_type = 'Event'
    src_meta_type = 'Dexterity Item'
    dst_portal_type = 'Event'
    dst_meta_type = None  # not used

    def migrate(self):
        # Only migrate items using old Schema
        if IEvent.providedBy(self.old) and hasattr(self.old, 'start_date'):
            ContentMigrator.migrate(self)

    def migrate_schema_fields(self):
        timezone = str(self.old.start_date.tzinfo) \
            if self.old.start_date.tzinfo \
            else default_timezone(fallback='UTC')

        self.new.start = datetime_fixer(self.old.start_date, timezone)
        self.new.end = datetime_fixer(self.old.end_date, timezone)

        if hasattr(self.old, 'location'):
            self.new.location = self.old.location
        if hasattr(self.old, 'attendees'):
            self.new.attendees = tuple(self.old.attendees.splitlines())
        if hasattr(self.old, 'event_url'):
            self.new.event_url = self.old.event_url
        if hasattr(self.old, 'contact_name'):
            self.new.contact_name = self.old.contact_name
        if hasattr(self.old, 'contact_email'):
            self.new.contact_email = self.old.contact_email
        if hasattr(self.old, 'contact_phone'):
            self.new.contact_phone = self.old.contact_phone
        if hasattr(self.old, 'text'):
            # Copy the entire richtext object, not just it's representation
            self.new.text = self.old.text


class DXEventMigrator(ContentMigrator):
    """Migrator for plone.app.event.dx events"""

    src_portal_type = 'plone.app.event.dx.event'
    src_meta_type = 'Dexterity Item'
    dst_portal_type = 'Event'
    dst_meta_type = None  # not used

    def migrate_schema_fields(self):
        self.new.start = datetime_fixer(self.old.start, self.old.timezone)
        self.new.end = datetime_fixer(self.old.end, self.old.timezone)
        self.new.whole_day = self.old.whole_day
        self.new.open_end = self.old.open_end
        self.new.recurrence = self.old.recurrence
        self.new.location = self.old.location
        self.new.attendees = self.old.attendees
        self.new.event_url = self.old.event_url
        self.new.contact_name = self.old.contact_name
        self.new.contact_email = self.old.contact_email
        self.new.contact_phone = self.old.contact_phone
        # The old behavior for the rich text field does not exist an more.
        # Look up the old value directly from the Annotation storage
        # Copy the entire richtext object, not just it's representation
        annotations = IAnnotations(self.old)
        old_text = annotations.get(
            'plone.app.event.dx.behaviors.IEventSummary.text', None)
        self.new.text = old_text


def get_old_class_name_string(obj):
    """Returns the current class name string."""
    return '{0}.{1}'.format(obj.__module__, obj.__class__.__name__)


def get_portal_type_name_string(obj):
    """Returns the klass-attribute of the fti."""
    fti = queryUtility(IDexterityFTI, name=obj.portal_type)
    if not fti:
        return False
    return fti.klass


def migrate_base_class_to_new_class(obj,
                                    indexes=None,
                                    old_class_name='',
                                    new_class_name='',
                                    migrate_to_folderish=False,
                                    ):
    if indexes is None:
        indexes = ['is_folderish', 'object_provides']
    if not old_class_name:
        old_class_name = get_old_class_name_string(obj)
    if not new_class_name:
        new_class_name = get_portal_type_name_string(obj)
        if not new_class_name:
            logger.warning(
                'The type {0} has no fti!'.format(obj.portal_type))
            return False

    was_item = not isinstance(obj, BTreeFolder2Base)
    if old_class_name != new_class_name:
        obj_id = obj.getId()
        module_name, class_name = new_class_name.rsplit('.', 1)
        module = importlib.import_module(module_name)
        new_class = getattr(module, class_name)

        # update obj class
        parent = obj.__parent__
        parent._delOb(obj_id)
        obj.__class__ = new_class
        parent._setOb(obj_id, obj)

    is_container = isinstance(obj, BTreeFolder2Base)

    if was_item and is_container or migrate_to_folderish and is_container:
        #  If Itemish becomes Folderish we have to update obj _tree
        BTreeFolder2Base._initBTrees(obj)

    # reindex
    obj.reindexObject(indexes)

    return True


def list_of_objects_with_changed_base_class(context):
    catalog = getToolByName(context, 'portal_catalog')
    for brain in catalog(object_provides=IDexterityContent.__identifier__):
        try:
            obj = brain.getObject()
        except (KeyError, NotFound):
            logger.warn('Object {0} not found'.format(brain.getPath()))
            continue
        if get_portal_type_name_string(obj) != get_old_class_name_string(obj):
            yield obj


def list_of_changed_base_class_names(context):
    """Returns list of class names that are not longer in portal_types."""
    changed_base_class_names = {}
    for obj in list_of_objects_with_changed_base_class(context):
        changed_base_class_name = get_old_class_name_string(obj)
        if changed_base_class_name not in changed_base_class_names:
            changed_base_class_names[changed_base_class_name] = 1
        else:
            changed_base_class_names[changed_base_class_name] += 1
    return changed_base_class_names



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletManager
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from Products.PythonScripts.PythonScript import PythonScript
from zope.component import getMultiAdapter
from zope.component import getUtility

import pkg_resources
import unittest


try:
    pkg_resources.get_distribution('plone.app.dexterity.behaviors.constraints')
except pkg_resources.DistributionNotFound:
    DEXTERITY_WITH_CONSTRAINS = False
else:
    DEXTERITY_WITH_CONSTRAINS = True


class PloneAppContenttypesContent(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpPloneSite(self, portal):
        # Necessary to set up some Plone stuff, such as Workflow.
        self.applyProfile(portal, 'plone.app.contenttypes:plone-content')


PLONE_APP_CONTENTTYPES_CONTENT_FIXTURE = PloneAppContenttypesContent()
PLONE_APP_CONTENTTYPES_CONTENT_INTEGRATION_TESTING = \
    IntegrationTesting(bases=(PLONE_APP_CONTENTTYPES_CONTENT_FIXTURE,),
                       name='PloneAppContenttypesContent:Integration')

# TODO Test for content translation.


class ContentProfileTestCase(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_CONTENT_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.portal_workflow = getToolByName(self.portal, 'portal_workflow')

    # #################### #
    #   front-page tests   #
    # #################### #

    def test_frontpage_was_created(self):
        # Was the object created?
        obj = self.portal['front-page']
        self.assertEqual(obj.portal_type, 'Document')

    def test_frontpage_is_default_page(self):
        # Has the object been set on the container as the default page?
        self.assertEqual(self.portal.default_page, 'front-page')

    def test_frontpage_is_published(self):
        # Has the content object been published?
        front_page = self.portal['front-page']
        current_state = self.portal_workflow.getInfoFor(
            front_page,
            'review_state')
        self.assertEqual(current_state, 'published')

    # ################# #
    #   Members tests   #
    # ################# #

    def test_Members_was_created(self):
        # Was the object created?
        obj = self.portal['Members']
        self.assertEqual(obj.portal_type, 'Folder')

    @unittest.skip('Replaced by new members-search-form')
    def test_Members__index_html(self):
        # Was the index_html script created?
        obj = self.portal['Members']['index_html']
        self.assertTrue(isinstance(obj, PythonScript))
        # It's outside the scope of this test to verify the contents of
        # the script are correct. Simply checking for existence should
        # be enough.

    def test_Members_portlets(self):
        # Have the right column portlet manager setting been added?
        members = self.portal['Members']
        manager = getUtility(IPortletManager, name='plone.rightcolumn')
        assignable_manager = getMultiAdapter((members, manager),
                                             ILocalPortletAssignmentManager)
        self.assertTrue(assignable_manager.getBlacklistStatus('context'))
        self.assertTrue(assignable_manager.getBlacklistStatus('group'))
        self.assertTrue(assignable_manager.getBlacklistStatus('content_type'))
        
    def test_Members_is_private(self):
        # Is the content object public?
        obj = self.portal['Members']
        current_state = self.portal_workflow.getInfoFor(obj, 'review_state')
        self.assertEqual(current_state, 'private')

    # ################ #
    #   events tests   #
    # ################ #

    def test_events_was_created(self):
        # Was the object created?
        events = self.portal['events']
        self.assertEqual(events.portal_type, 'Folder')
        # Was the contained collection created?
        collection = events['aggregator']
        self.assertEqual(collection.portal_type, 'Collection')

    def test_events_default_page(self):
        # Has the object been set on the container as the default page?
        self.assertEqual(self.portal['events'].default_page, 'aggregator')

    def test_events_is_published(self):
        # Has the content object been published?
        events = self.portal['events']
        current_state = self.portal_workflow.getInfoFor(events, 'review_state')
        self.assertEqual(current_state, 'published')

    @unittest.skipUnless(
        DEXTERITY_WITH_CONSTRAINS,
        'Dexterity constraints are not present')
    def test_events_allowable_types(self):
        events = self.portal['events']
        behavior = ISelectableConstrainTypes(events)
        types = ['Event']
        self.assertEqual(types, behavior.getImmediatelyAddableTypes())

    # ############## #
    #   news tests   #
    # ############## #

    def test_news_was_created(self):
        # Was the object created?
        news = self.portal['news']
        self.assertEqual(news.portal_type, 'Folder')
        # Was the contained collection created?
        collection = news['aggregator']
        self.assertEqual(collection.portal_type, 'Collection')

    def test_news_default_page(self):
        # Has the object been set on the container as the default page?
        self.assertEqual(self.portal['news'].default_page, 'aggregator')

    def test_news_is_published(self):
        # Has the content object been published?
        news = self.portal['news']
        current_state = self.portal_workflow.getInfoFor(news, 'review_state')
        self.assertEqual(current_state, 'published')

    @unittest.skipUnless(
        DEXTERITY_WITH_CONSTRAINS,
        'Dexterity constraints are not present')
    def test_news_allowable_types(self):
        news = self.portal['news']
        behavior = ISelectableConstrainTypes(news)
        types = ['News Item']
        self.assertEqual(types, behavior.getImmediatelyAddableTypes())

    def test_news_aggregator_settings(self):
        # Has the news aggregator (Collection) been set up?
        query = [dict(i=u'portal_type',
                      o=u'plone.app.querystring.operation.selection.any',
                      v=[u'News Item']),
                 dict(i=u'review_state',
                      o=u'plone.app.querystring.operation.selection.any',
                      v=[u'published']),
                 ]
        collection = self.portal['news']['aggregator']
        self.assertEqual(collection.sort_on, u'effective')
        self.assertEqual(collection.sort_reversed, True)
        self.assertItemsEqual(collection.query, query)
        self.assertEqual(collection.getLayout(), 'summary_view')



# -*- coding: utf-8 -*-
from plone.app.contenttypes import _
from plone.app.textfield import RichText as RichTextField
from plone.app.z3cform.widget import RichTextFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.autoform.view import WidgetsView
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IRichText(model.Schema):

    text = RichTextField(
        title=_(u'Text'),
        description=u'',
        required=False,
    )
    form.widget('text', RichTextFieldWidget)
    model.primary('text')


@implementer(IRichText)
@adapter(IDexterityContent)
class RichText(object):

    def __init__(self, context):
        self.context = context


class WidgetView(WidgetsView):
    schema = IRichText



# -*- coding: utf-8 -*-
from datetime import datetime
from plone.app.contenttypes.migration.field_migrators import migrate_filefield
from plone.app.contenttypes.migration.field_migrators import migrate_imagefield
from plone.app.contenttypes.migration.field_migrators import migrate_simplefield  # noqa
from plone.app.contenttypes.migration.utils import installTypeIfNeeded
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_MIGRATION_FUNCTIONAL_TESTING  # noqa
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_MIGRATION_TESTING  # noqa
from plone.app.testing import applyProfile
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.testing.z2 import Browser
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode

import os.path
import pytz
import transaction
import unittest


class MigrateFieldsTest(unittest.TestCase):

    layer = PLONE_APP_CONTENTTYPES_MIGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def tearDown(self):
        try:
            applyProfile(self.portal, 'plone.app.contenttypes:uninstall')
        except KeyError:
            pass

    def get_test_image_data(self):
        test_image_path = os.path.join(os.path.dirname(__file__), 'image.png')
        with open(test_image_path, 'rb') as test_image_file:
            test_image_data = test_image_file.read()
        return test_image_data

    def get_test_file_data(self):
        test_file_path = os.path.join(os.path.dirname(__file__), 'file.pdf')
        with open(test_file_path, 'rb') as test_file:
            test_file_data = test_file.read()
        return test_file_data

    def test_migrate_stringfield(self):
        # create content
        at_document_id = self.portal.invokeFactory(
            'Document',
            'foo',
            title='Foo document',
        )
        # register p.a.contenttypes profile
        applyProfile(self.portal, 'plone.app.contenttypes:default')
        dx_document_id = self.portal.invokeFactory(
            'Document',
            'bar',
            title='Bar document',
        )
        at_document = self.portal[at_document_id]
        dx_document = self.portal[dx_document_id]
        migrate_simplefield(at_document, dx_document, 'title', 'title')
        self.assertEqual(dx_document.Title(), at_document.Title())

    def test_migrate_richtextfield(self):
        # create content
        at_document_id = self.portal.invokeFactory(
            'Document',
            'foo',
            title='Foo document',
            text='Some foo html text',
        )
        # register p.a.contenttypes profile
        applyProfile(self.portal, 'plone.app.contenttypes:default')
        dx_document_id = self.portal.invokeFactory(
            'Document',
            'bar',
            title='Bar document',
        )
        at_document = self.portal[at_document_id]
        dx_document = self.portal[dx_document_id]
        self.assertEqual(dx_document.text, None)
        migrate_simplefield(at_document, dx_document, 'text', 'text')
        self.assertEqual(dx_document.text, at_document.getText())

    def test_migrate_listfield(self):
        # create content
        at_document_id = self.portal.invokeFactory(
            'Document',
            'foo',
            title='Foo document',
            subject=['aaa', 'bbb'],
        )
        # register p.a.contenttypes profile
        applyProfile(self.portal, 'plone.app.contenttypes:default')
        dx_document_id = self.portal.invokeFactory(
            'Document',
            'bar',
            title='Bar document',
        )
        at_document = self.portal[at_document_id]
        dx_document = self.portal[dx_document_id]
        migrate_simplefield(at_document, dx_document, 'subject', 'subject',)
        self.assertEqual(dx_document.Subject(), at_document.Subject())

    def test_migrate_imagefield(self):
        test_image_data = self.get_test_image_data()
        at_newsitem_id = self.portal.invokeFactory(
            'News Item',
            'foo',
            title='Foo news',
            image=test_image_data,
        )
        # register p.a.contenttypes profile
        applyProfile(self.portal, 'plone.app.contenttypes:default')
        dx_newsitem_id = self.portal.invokeFactory(
            'News Item',
            'bar',
            title='Bar news',
        )
        at_newsitem = self.portal[at_newsitem_id]
        dx_newsitem = self.portal[dx_newsitem_id]
        self.assertEqual(dx_newsitem.image, None)
        migrate_imagefield(at_newsitem, dx_newsitem, 'image', 'image')
        self.assertEqual(dx_newsitem.image.contentType, 'image/png')
        self.assertEqual(dx_newsitem.image.data, test_image_data)

    def test_migrate_filefield(self):
        test_file_data = self.get_test_file_data()
        at_file_id = self.portal.invokeFactory(
            'File',
            'foo',
            title='Foo file',
            file=test_file_data,
        )
        # register p.a.contenttypes profile
        applyProfile(self.portal, 'plone.app.contenttypes:default')
        dx_file_id = self.portal.invokeFactory(
            'File',
            'bar',
            title='Bar file',
        )
        at_file = self.portal[at_file_id]
        dx_file = self.portal[dx_file_id]
        self.assertEqual(dx_file.file, None)
        migrate_filefield(at_file, dx_file, 'file', 'file')
        self.assertEqual(dx_file.file.data, test_file_data)


class MigrateCustomATTest(unittest.TestCase):

    layer = PLONE_APP_CONTENTTYPES_MIGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def tearDown(self):
        try:
            applyProfile(self.portal, 'plone.app.contenttypes:uninstall')
        except KeyError:
            pass

    def createCustomATDocument(self, id, parent=None):
        from Products.Archetypes.atapi import StringField, TextField
        from Products.ATContentTypes.interfaces import IATDocument
        from archetypes.schemaextender.interfaces import ISchemaExtender
        from archetypes.schemaextender.field import ExtensionField
        from zope.component import getGlobalSiteManager
        from zope.interface import implementer

        # create schema extension
        class ExtensionTextField(ExtensionField, TextField):
            """ derivative of text for extending schemas """

        class ExtensionStringField(ExtensionField, StringField):
            """ derivative of text for extending schemas """

        @implementer(ISchemaExtender)
        class SchemaExtender(object):
            fields = [
                ExtensionTextField('textExtended',
                                   ),
                ExtensionStringField('stringExtended',
                                     ),
            ]

            def __init__(self, context):
                self.context = context

            def getFields(self):
                return self.fields

        # register adapter
        gsm = getGlobalSiteManager()
        gsm.registerAdapter(SchemaExtender, (IATDocument,), ISchemaExtender)

        # create content
        container = parent or self.portal
        container.invokeFactory(
            'Document',
            id,
            title='Foo document',
            stringExtended='foo text',
            textExtended='foo extended rich text',
        )
        at_document = container[id]

        # unregister adapter assure test isolation
        gsm.unregisterAdapter(required=[IATDocument], provided=ISchemaExtender)

        return at_document

    def test_migrate_extended_document(self):
        from plone.app.contenttypes.migration.migration import migrateCustomAT
        from plone.app.contenttypes.interfaces import INewsItem
        at_document = self.createCustomATDocument('foo-document')
        # install pac but only install News Items
        portal_setup = getToolByName(self.portal, 'portal_setup')
        portal_setup.runAllImportStepsFromProfile(
            'profile-plone.app.contenttypes:default',
            blacklisted_steps=['typeinfo'],
        )
        installTypeIfNeeded('News Item')
        fields_mapping = (
            {'AT_field_name': 'textExtended',
             'AT_field_type': 'Products.Archetypes.Field.TextField',
             'DX_field_name': 'text',
             'DX_field_type': 'RichText', },
            {'AT_field_name': 'stringExtended',
             'AT_field_type': 'StringField',
             'DX_field_name': 'title',
             'DX_field_type': 'StringField', },
        )
        # migrate extended AT Document to default DX News Item
        migrateCustomAT(
            fields_mapping, src_type='Document', dst_type='News Item')
        dx_newsitem = self.portal['foo-document']
        self.assertTrue(INewsItem.providedBy(dx_newsitem))
        self.assertTrue(dx_newsitem is not at_document)
        self.assertEqual(at_document.textExtended(), dx_newsitem.text.raw)
        self.assertEqual(at_document.stringExtended, dx_newsitem.title)

    def test_migrate_atevent_to_dxnewsitem(self):
        """Tests the custom migration by migrating a default type. It is not
        meant to be used this way but is a nice way to test the migrations.
        During this migration the old event fti is still present.
        """
        from DateTime import DateTime
        from plone.app.contenttypes.migration.migration import migrateCustomAT
        from plone.app.contenttypes.interfaces import INewsItem

        # create an ATEvent
        self.portal.invokeFactory('Event', 'event')
        at_event = self.portal['event']

        # Date
        at_event.getField('startDate') \
                .set(at_event, DateTime('2013-02-03 12:00'))
        at_event.getField('endDate') \
                .set(at_event, DateTime('2013-04-05 13:00'))

        # Contact
        at_event.getField('contactPhone').set(at_event, '123456789')
        at_event.getField('contactEmail').set(at_event, 'dummy@email.com')
        at_event.getField('contactName').set(at_event, u'Näme')

        # URL
        at_event.getField('eventUrl').set(at_event, 'http://www.plone.org')

        # Attendees
        at_event.getField('attendees').set(at_event, ('You', 'Me'))

        # Text
        at_event.setText('Tütensuppe')
        at_event.setContentType('text/plain')

        oldTZ = os.environ.get('TZ', None)
        os.environ['TZ'] = 'Asia/Tbilisi'

        # install pac but only install News Items
        portal_setup = getToolByName(self.portal, 'portal_setup')
        portal_setup.runAllImportStepsFromProfile(
            'profile-plone.app.contenttypes:default',
            blacklisted_steps=['typeinfo'],
        )
        installTypeIfNeeded('News Item')
        fields_mapping = (
            {'AT_field_name': 'text',
             'AT_field_type': 'Products.Archetypes.Field.TextField',
             'DX_field_name': 'text',
             'DX_field_type': 'RichText', },
            {'AT_field_name': 'contactName',
             'AT_field_type': 'StringField',
             'DX_field_name': 'image_caption',
             'DX_field_type': 'StringField', },
        )
        # migrate ATCTEvent to default DX News Item
        migrateCustomAT(fields_mapping, src_type='Event', dst_type='News Item')
        if oldTZ:
            os.environ['TZ'] = oldTZ
        else:
            del os.environ['TZ']

        dx_newsitem = self.portal['event']
        self.assertTrue(INewsItem.providedBy(dx_newsitem))
        self.assertTrue(dx_newsitem is not at_event)
        self.assertEqual(
            safe_unicode(at_event.getText()),
            dx_newsitem.text.output)
        self.assertEqual(
            at_event.contactName,
            dx_newsitem.image_caption)

    def test_migrate_atevent_to_dxevent(self):
        """Tests the custom migration by migrating a default type. It is not
        meant to be used this way but is a nice way to test the migrations.
        During this migration the event fti is already replaced by the dx one.
        """
        from DateTime import DateTime
        from plone.app.contenttypes.migration.migration import migrateCustomAT
        from plone.app.contenttypes.interfaces import IEvent

        # create an ATEvent
        self.portal.invokeFactory('Event', 'event')
        at_event = self.portal['event']

        # Date
        FORMAT = '%Y-%m-%d %H:%M'
        start = '2013-02-03 12:15'
        end = '2013-04-05 13:45'
        at_event.getField('startDate').set(at_event, DateTime(start))
        at_event.getField('endDate').set(at_event, DateTime(end))

        # Contact
        at_event.getField('contactPhone').set(at_event, '123456789')
        at_event.getField('contactEmail').set(at_event, 'dummy@email.com')
        at_event.getField('contactName').set(at_event, u'Näme')

        # URL
        at_event.getField('eventUrl').set(at_event, 'http://www.plone.org')

        # Attendees
        at_event.getField('attendees').set(at_event, ('Yöu', 'Me'))

        # Text
        at_event.setText('Tütensuppe')
        at_event.setContentType('text/plain')

        oldTZ = os.environ.get('TZ', None)
        TZ = 'Asia/Tbilisi'
        os.environ['TZ'] = TZ
        timezone = pytz.timezone(TZ)

        # install pac but only install Event
        portal_setup = getToolByName(self.portal, 'portal_setup')
        portal_setup.runAllImportStepsFromProfile(
            'profile-plone.app.contenttypes:default',
            blacklisted_steps=['typeinfo'],
        )
        installTypeIfNeeded('Event')
        fields_mapping = (
            {'AT_field_name': 'startDate',
             'AT_field_type': 'Products.Archetypes.Field.DateTimeField',
             'DX_field_name': 'start',
             'DX_field_type': 'Datetime', },
            {'AT_field_name': 'endDate',
             'AT_field_type': 'Products.Archetypes.Field.DateTimeField',
             'DX_field_name': 'end',
             'DX_field_type': 'Datetime', },
            {'AT_field_name': 'text',
             'AT_field_type': 'Products.Archetypes.Field.TextField',
             'DX_field_name': 'text',
             'DX_field_type': 'RichText', },
            {'AT_field_name': 'eventUrl',
             'AT_field_type': 'Products.Archetypes.Field.StringField',
             'DX_field_name': 'event_url',
             'DX_field_type': 'StringField', },
            {'AT_field_name': 'contactEmail',
             'AT_field_type': 'Products.Archetypes.Field.StringField',
             'DX_field_name': 'contact_email',
             'DX_field_type': 'StringField', },
            {'AT_field_name': 'contactName',
             'AT_field_type': 'Products.Archetypes.Field.StringField',
             'DX_field_name': 'contact_name',
             'DX_field_type': 'StringField', },
            {'AT_field_name': 'contactPhone',
             'AT_field_type': 'Products.Archetypes.Field.StringField',
             'DX_field_name': 'contact_phone',
             'DX_field_type': 'StringField', },
            {'AT_field_name': 'attendees',
             'AT_field_type': 'Products.Archetypes.Field.LinesField',
             'DX_field_name': 'attendees',
             'DX_field_type': 'Tuple', },
        )
        # migrate ATEvent to new default Event
        migrateCustomAT(fields_mapping, src_type='Event', dst_type='Event')
        dx_event = self.portal['event']
        self.assertTrue(IEvent.providedBy(dx_event))
        self.assertTrue(dx_event is not at_event)
        self.assertEqual(safe_unicode(
            at_event.getText()), dx_event.text.output)
        self.assertEqual(at_event.eventUrl, dx_event.event_url)
        self.assertEqual(at_event.contactEmail, dx_event.contact_email)
        self.assertEqual(at_event.contactName, dx_event.contact_name)
        self.assertEqual(at_event.contactPhone, dx_event.contact_phone)
        self.assertEqual(at_event.attendees, dx_event.attendees)
        self.assertEqual(
            dx_event.start,
            timezone.localize(datetime.strptime(start, FORMAT)))
        self.assertEqual(
            dx_event.end, timezone.localize(datetime.strptime(end, FORMAT)))
        if oldTZ:
            os.environ['TZ'] = oldTZ
        else:
            del os.environ['TZ']

    def test_migration_with_custom_fieldmigrator(self):
        """Migrate a ATDocument to a DXNewsItem using a custom modifier"""
        from plone.app.contenttypes.interfaces import INewsItem
        from plone.app.contenttypes.migration.migration import migrateCustomAT
        at_document = self.createCustomATDocument('foo-document')
        at_text = (
            u'Some | field is | pipe-delimited | in the field\n'
        )
        at_document.setText(at_text)
        # install pac but only install News Items
        portal_setup = getToolByName(self.portal, 'portal_setup')
        portal_setup.runAllImportStepsFromProfile(
            'profile-plone.app.contenttypes:default',
            blacklisted_steps=['typeinfo'],
        )
        installTypeIfNeeded('News Item')
        fields_mapping = (
            {'AT_field_name': 'text',
             'DX_field_name': 'creators',
             'field_migrator': some_field_migrator},
        )
        migrateCustomAT(
            fields_mapping, src_type='Document', dst_type='News Item')

        dx_newsitem = self.portal['foo-document']
        self.assertTrue(INewsItem.providedBy(dx_newsitem))
        self.assertTrue(dx_newsitem is not at_document)
        self.assertEqual(4, len(dx_newsitem.creators))
        self.assertEqual(at_document.Title(), dx_newsitem.title)


def some_field_migrator(src_obj, dst_obj, src_fieldname, dst_fieldname):
    """Custom field_migrator

    A simple example that transforms the value of a pipe-delimited richtext
    to a tuple.
    """
    field = src_obj.getField(src_fieldname)
    at_value = field.get(src_obj)
    at_value = at_value.replace('<p>', '').replace('</p>', '')
    dx_value = [safe_unicode(i) for i in at_value.split('|')]
    setattr(dst_obj, dst_fieldname, tuple(dx_value))


class CustomMigrationFunctionalTests(unittest.TestCase):

    layer = PLONE_APP_CONTENTTYPES_MIGRATION_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer['app']
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.request['ACTUAL_URL'] = self.portal.absolute_url()
        self.request['URL'] = self.portal.absolute_url()
        self.catalog = getToolByName(self.portal, 'portal_catalog')
        self.portal.acl_users.userFolderAddUser('admin',
                                                'secret',
                                                ['Manager'],
                                                [])
        login(self.portal, 'admin')
        self.portal.portal_workflow.setDefaultChain(
            'simple_publication_workflow')
        self.portal_url = self.portal.absolute_url()

        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic {0}:{1}'.format(SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
        )

    def tearDown(self):
        try:
            applyProfile(self.portal, 'plone.app.contenttypes:uninstall')
        except KeyError:
            pass

    def test_custom_migration_form(self):
        """Basic test for the custom_migration form.
        Field-mapping only works with javascript enabled so we migrate
        only the content but not the fields.
        """
        # add some at content
        self.portal.invokeFactory('Document', 'doc1')
        self.portal.invokeFactory('Event', 'event1')
        self.portal.event1.setTitle(u'Ein Törmin')
        self.portal.event1.setDescription(u'Wänn?')
        self.portal.doc1.setTitle(u'Ein Döcument')
        self.portal.doc1.setDescription(u'Sö was')
        transaction.commit()
        self.browser.open('{0}/@@pac_installer'.format(self.portal_url))
        self.browser.getControl('Install').click()
        # open custom-migration-form
        self.browser.open('{0}/@@custom_migration'.format(self.portal_url))
        results = self.browser.contents
        self.assertIn('Custom types migration control panel', results)
        self.assertIn('<input type="hidden" name="Document:list" value="text__type__Products.Archetypes.Field.TextField" />', results)  # noqa
        self.assertEqual(self.browser.getControl(name='dx_select_Document').value, [''])  # noqa
        # chose to migrate to Link
        self.browser.getControl(name='dx_select_Document').value = ['Link']
        self.assertIn('<input type="hidden" name="Event:list" value="startDate__type__Products.Archetypes.Field.DateTimeField" />', results)  # noqa
        # chose to migrate to Link
        self.browser.getControl(name='dx_select_Event').value = ['Link']
        # run migration
        self.browser.getControl(name='form.button.Migrate').click()
        results = self.browser.contents
        self.assertIn(
            'Migration applied successfully for 1 "Document" items.',
            results,
        )
        self.assertIn(
            'Migration applied successfully for 1 "Event" items.',
            results,
        )
        self.assertIn('No content to migrate.', results)
        link1 = self.portal['doc1']
        self.assertEqual(link1.portal_type, 'Link')
        self.assertEqual(link1.title, u'Ein D\xf6cument')
        self.assertEqual(link1.description, u'S\xf6 was')
        self.assertEqual(self.portal['event1'].portal_type, 'Link')
        # we did not migrate the fields so lets find out if it is a real Link
        link1.remote_url = 'http://www.starzel.de'
        view = link1()
        self.assertIn(u'<h1 class="documentFirstHeading">Ein D\xf6cument</h1>', view)  # noqa
        self.assertIn(u'The link address is:</span>\n            <a href="http://www.starzel.de">http://www.starzel.de</a>', view)  # noqa

    def test_custom_migration_form_for_types_with_spaces(self):
        """Basic test for the custom_migration form.
        Field-mapping only works with javascript enabled so we migrate
        only the content but not the fields.
        """
        # add some at content
        self.portal.invokeFactory('News Item', 'news1')
        self.portal.invokeFactory('Event', 'event1')
        self.portal.event1.setTitle(u'Ein Törmin')
        self.portal.event1.setDescription(u'Wänn?')
        self.portal.news1.setTitle(u'Ein News Item')
        self.portal.news1.setDescription(u'Sö was')
        transaction.commit()
        self.browser.open('{0}/@@pac_installer'.format(self.portal_url))
        self.browser.getControl('Install').click()
        # open custom-migration-form
        self.browser.open('{0}/@@custom_migration'.format(self.portal_url))
        results = self.browser.contents
        self.assertIn('Custom types migration control panel', results)
        self.assertIn('<input type="hidden" name="News_space_Item:list" value="text__type__Products.Archetypes.Field.TextField" />', results)  # noqa
        self.assertEqual(self.browser.getControl(name='dx_select_News_space_Item').value, [''])  # noqa
        # chose to migrate to Link
        self.browser.getControl(name='dx_select_News_space_Item').value = ['Link']  # noqa
        self.assertIn('<input type="hidden" name="Event:list" value="startDate__type__Products.Archetypes.Field.DateTimeField" />', results)  # noqa
        # chose to migrate to Link
        self.browser.getControl(name='dx_select_Event').value = ['Link']
        # run migration
        self.browser.getControl(name='form.button.Migrate').click()
        results = self.browser.contents
        self.assertIn(
            'Migration applied successfully for 1 "News Item" items.',
            results,
        )
        self.assertIn(
            'Migration applied successfully for 1 "Event" items.',
            results,
        )
        self.assertIn('No content to migrate.', results)
        link1 = self.portal['news1']
        self.assertEqual(link1.portal_type, 'Link')
        self.assertEqual(link1.title, u'Ein News Item')
        self.assertEqual(link1.description, u'S\xf6 was')
        self.assertEqual(self.portal['event1'].portal_type, 'Link')
        # we did not migrate the fields so lets find out if it is a real Link
        link1.remote_url = 'http://www.starzel.de'
        view = link1()
        self.assertIn(u'<h1 class="documentFirstHeading">Ein News Item</h1>', view)  # noqa
        self.assertIn(u'The link address is:</span>\n            <a href="http://www.starzel.de">http://www.starzel.de</a>', view)  # noqa



# -*- coding: utf-8 -*-
"""Patches used for migrations. These patches are applied before and removed
after running the migration.
"""
from plone.dexterity.content import DexterityContent
from plone.registry.interfaces import IRegistry
from Products.Archetypes.ExtensibleMetadata import ExtensibleMetadata
from Products.CMFCore.interfaces import IPropertiesTool
from Products.CMFPlone.DublinCore import DefaultDublinCoreImpl
from Products.CMFPlone.interfaces import IEditingSchema
from Products.contentmigration.utils import patch
from Products.contentmigration.utils import undoPatch
from Products.PluginIndexes.common.UnIndex import _marker
from Products.PluginIndexes.UUIDIndex.UUIDIndex import UUIDIndex
from zope.component import getUtility
from zope.component import queryUtility

import os


def pass_fn(*args, **kwargs):
    """Empty function used for patching."""
    pass


def patched_index_object(*args, **kwargs):
    """Patched Products.ZCTextIndex.ZCTextIndex.ZCTextIndex.index_object"""
    return 0


# Prevent UUID Error-Messages when migrating folders.
# Products.PluginIndexes.UUIDIndex.UUIDIndex.UUIDIndex.insertForwardIndexEntry
def patched_insertForwardIndexEntry(self, entry, documentId):
    """Take the entry provided and put it in the correct place
    in the forward index.
    """
    if entry is None:
        return

    old_docid = self._index.get(entry, _marker)
    if old_docid is _marker:
        self._index[entry] = documentId
        self._length.change(1)


def patch_before_migration(patch_searchabletext=False):
    """Patch various things that make migration harder."""
    # Switch linkintegrity off
    ptool = queryUtility(IPropertiesTool)
    site_props = getattr(ptool, 'site_properties', None)
    if site_props and site_props.hasProperty(
            'enable_link_integrity_checks'):
        link_integrity = site_props.getProperty(
            'enable_link_integrity_checks', False)
        site_props.manage_changeProperties(
            enable_link_integrity_checks=False)
    else:
        # Plone 5
        registry = getUtility(IRegistry)
        editing_settings = registry.forInterface(
            IEditingSchema, prefix='plone')
        link_integrity = editing_settings.enable_link_integrity_checks
        editing_settings.enable_link_integrity_checks = False

    # Patch notifyModified to prevent setModificationDate() on changes
    # notifyModified lives in several places and is also used on folders
    # when their content changes.
    # So when we migrate Documents before Folders the folders
    # ModifiedDate gets changed
    PATCH_NOTIFY = [
        DexterityContent,
        DefaultDublinCoreImpl,
        ExtensibleMetadata
    ]
    for klass in PATCH_NOTIFY:
        patch(klass, 'notifyModified', pass_fn)

    # Disable queueing of indexing/reindexing/unindexing
    queue_indexing = os.environ.get('CATALOG_OPTIMIZATION_DISABLED', None)
    os.environ['CATALOG_OPTIMIZATION_DISABLED'] = '1'

    # Patch UUIDIndex
    patch(
        UUIDIndex,
        'insertForwardIndexEntry',
        patched_insertForwardIndexEntry)

    # Patch SearchableText index
    if patch_searchabletext:
        patch_indexing_at_blobs()
        patch_indexing_dx_blobs()

    return link_integrity, queue_indexing, patch_searchabletext


def undo_patch_after_migration(link_integrity=True,
                               queue_indexing=None,
                               patch_searchabletext=False,
                               ):
    """Revert to the original state."""

    # Switch linkintegrity back to what it was before migrating
    ptool = queryUtility(IPropertiesTool)
    site_props = getattr(ptool, 'site_properties', None)
    if site_props and site_props.hasProperty(
            'enable_link_integrity_checks'):
        site_props.manage_changeProperties(
            enable_link_integrity_checks=link_integrity
        )
    else:
        # Plone 5
        registry = getUtility(IRegistry)
        editing_settings = registry.forInterface(
            IEditingSchema, prefix='plone')
        editing_settings.enable_link_integrity_checks = link_integrity

    # Switch on setModificationDate on changes
    PATCH_NOTIFY = [
        DexterityContent,
        DefaultDublinCoreImpl,
        ExtensibleMetadata
    ]
    for klass in PATCH_NOTIFY:
        undoPatch(klass, 'notifyModified')

    # Reset queueing of indexing/reindexing/unindexing
    if queue_indexing is not None:
        os.environ['CATALOG_OPTIMIZATION_DISABLED'] = queue_indexing
    else:
        del os.environ['CATALOG_OPTIMIZATION_DISABLED']

    # Unpatch UUIDIndex
    undoPatch(UUIDIndex, 'insertForwardIndexEntry')

    # Unpatch SearchableText index
    if patch_searchabletext:
        unpatch_indexing_at_blobs()
        unpatch_indexing_dx_blobs()


def patch_indexing_at_blobs():
    from plone.app.blob.content import ATBlob
    patch(ATBlob, 'getIndexValue', pass_fn)


def unpatch_indexing_at_blobs():
    from plone.app.blob.content import ATBlob
    undoPatch(ATBlob, 'getIndexValue')


def patch_indexing_dx_blobs():
    from Products.ZCTextIndex.ZCTextIndex import ZCTextIndex
    patch(ZCTextIndex, 'index_object', patched_index_object)


def unpatch_indexing_dx_blobs():
    from Products.ZCTextIndex.ZCTextIndex import ZCTextIndex
    undoPatch(ZCTextIndex, 'index_object')



# -*- coding: utf-8 -*-
from datetime import datetime
from plone.app.z3cform.converters import LinkWidgetDataConverter
from plone.app.z3cform.widget import LinkWidget
from plone.app.contenttypes.interfaces import ILink
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING  # noqa
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING  # noqa
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.dexterity.interfaces import IDexterityFTI
from plone.testing.z2 import Browser
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from zope.component import createObject
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.event import notify
from zope.interface import alsoProvides
from zope.schema import TextLine
from zope.traversing.interfaces import BeforeTraverseEvent

import unittest


class LinkIntegrationTest(unittest.TestCase):

    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.request['ACTUAL_URL'] = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ['Contributor'])

    def test_schema(self):
        fti = queryUtility(
            IDexterityFTI,
            name='Link')
        schema = fti.lookupSchema()
        self.assertEqual(schema.getName(), 'plone_0_Link')

    def test_fti(self):
        fti = queryUtility(
            IDexterityFTI,
            name='Link'
        )
        self.assertNotEquals(None, fti)

    def test_factory(self):
        fti = queryUtility(
            IDexterityFTI,
            name='Link'
        )
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(ILink.providedBy(new_object))

    def test_adding(self):
        self.portal.invokeFactory(
            'Link',
            'doc1'
        )
        self.assertTrue(ILink.providedBy(self.portal['doc1']))


class LinkViewIntegrationTest(unittest.TestCase):

    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.request['ACTUAL_URL'] = self.portal.absolute_url()
        self.response = self.request.response
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Link', 'link')
        link = self.portal['link']
        link.title = 'My Link'
        link.description = 'This is my link.'
        self.link = link
        self.request.set('URL', link.absolute_url())
        self.request.set('ACTUAL_URL', link.absolute_url())
        alsoProvides(self.request, IPloneFormLayer)

        # setup manually the correct browserlayer, see:
        # https://dev.plone.org/ticket/11673
        notify(BeforeTraverseEvent(self.portal, self.request))

    def test_link_redirect_view(self):
        view = self.link.restrictedTraverse('@@view')
        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)
        self.assertTrue('My Link' in view())
        self.assertTrue('This is my link.' in view())

    def test_link_redirect_view_external_url(self):
        self.link.remoteUrl = 'http://www.plone.org'
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect('http://www.plone.org')

    def test_link_redirect_view_absolute_path(self):
        self.link.remoteUrl = '/plone/my-folder/my-item'
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect('http://nohost/plone/my-folder/my-item')

    def test_link_redirect_view_relative_path(self):
        self.link.remoteUrl = '../my-item'
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        # The following URL will be redirected to:
        # "http://nohost/plone/my-item"
        self._assert_redirect('http://nohost/plone/link/../my-item')

    def test_link_redirect_view_path_with_variable(self):
        self.link.remoteUrl = '${navigation_root_url}/my-folder/my-item'
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect('http://nohost/plone/my-folder/my-item')

        # Should give the same result with ${portal_url}
        self.link.remoteUrl = '${portal_url}/my-folder/my-item'
        self.assertTrue(view())
        self._assert_redirect('http://nohost/plone/my-folder/my-item')

    def test_link_redirect_view_path_with_variable_and_parameters(self):
        # https://github.com/plone/plone.app.contenttypes/issues/457
        self.link.remoteUrl = '${portal_url}/@@search?SearchableText=Plone'
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect('http://nohost/plone/@@search?SearchableText=Plone')

    def test_mailto_type(self):
        self.link.remoteUrl = 'mailto:stress@test.us'
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="mailto:stress@test.us"', rendered)
        self._assert_response_OK()

    def test_tel_type(self):
        self.link.remoteUrl = 'tel:123'
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="tel:123"', rendered)
        self._assert_response_OK()

    def test_callto_type(self):
        self.link.remoteUrl = 'callto:123'
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="callto:123"', rendered)
        self._assert_response_OK()

    def test_webdav_type(self):
        self.link.remoteUrl = 'webdav://web.site/resource'
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn(
            'href="webdav://web.site/resource"',
            rendered
        )
        self._assert_response_OK()

    def test_caldav_type(self):
        self.link.remoteUrl = 'caldav://calendar.site/resource'
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn(
            'href="caldav://calendar.site/resource"',
            rendered
        )
        self._assert_response_OK()

    def test_file_type(self):
        self.link.remoteUrl = 'file:///some/file/on/your/system'
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        self.assertTrue(view())
        self._assert_redirect(self.link.remoteUrl)

    def test_ftp_type(self):
        self.link.remoteUrl = 'ftp://thereIsNoSuchDomain.isThere{0}'.format(
            datetime.now().isoformat()
        )
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        self.assertTrue(view())
        self._assert_redirect(self.link.remoteUrl)

    def _publish(self, obj):
        portal_workflow = getToolByName(self.portal, 'portal_workflow')
        portal_workflow.doActionFor(obj, 'publish')

    def _assert_redirect(self, url):
        self.assertEqual(self.response.status, 302)
        self.assertEqual(self.response.headers['location'], url)

    def _assert_response_OK(self):
        self.assertEqual(self.response.status, 200)

    def _get_link_redirect_view(self, obj):
        return getMultiAdapter((obj, self.request), name='link_redirect_view')


class LinkFunctionalTest(unittest.TestCase):

    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer['app']
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic {0}:{1}'.format(SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
        )

    def test_add_link(self):
        self.browser.open(self.portal_url)
        self.browser.getLink('Link').click()
        self.browser.getControl(name='form.widgets.IDublinCore.title')\
            .value = 'My link'
        self.browser.getControl(name='form.widgets.IDublinCore.description')\
            .value = 'This is my link.'
        self.browser.getControl(name='form.widgets.IShortName.id')\
            .value = 'my-special-link'
        self.browser.getControl(name='form.widgets.remoteUrl.external')\
            .value = 'https://plone.org'
        self.browser.getControl('Save').click()

        self.assertTrue(self.browser.url.endswith('my-special-link/view'))
        self.assertTrue('My link' in self.browser.contents)
        self.assertTrue('This is my link' in self.browser.contents)


class LinkWidgetIntegrationTest(unittest.TestCase):

    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    default_result = {'internal': u'',
                      'external': u'',
                      'email': u'',
                      'email_subject': u''}

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.request['ACTUAL_URL'] = self.portal.absolute_url()
        self.response = self.request.response
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        lid = self.portal.invokeFactory('Link', 'link',
                                        title='My Link',
                                        description="This is my link.")
        self.link = self.portal[lid]
        self.link_field = TextLine()
        self.widget = LinkWidget(self.request)

    def test_dc_towidget_external(self):
        url = 'http://www.example.org'
        self.link.remoteUrl = url

        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected['external'] = url
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_towidget_internal(self):
        self.portal.invokeFactory('Document', 'doc1',
                                  title='A document',
                                  description="This is a document.")
        doc1 = self.portal['doc1']
        url = doc1.absolute_url()
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected['internal'] = IUUID(doc1)
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_towidget_mail(self):
        url = u'mailto:foo@.example.org'
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected['email'] = url[7:]   # mailto is cut
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_towidget_mail_subject(self):
        url = 'mailto:foo@.example.org?subject=A subject'
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected['email'] = u'foo@.example.org'
        expected['email_subject'] = u'A subject'
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_illegal(self):
        url = 'foo'
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected['external'] = url
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_var(self):
        url = '${portal_url}/foo'
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected['external'] = url
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_var_replacement_in_view(self):
        view = getMultiAdapter(
            (self.link, self.request),
            name='link_redirect_view'
        )

        self.link.remoteUrl = '${portal_url}'
        self.assertEqual(view.url(), '/plone')
        self.assertEqual(view.absolute_target_url(), 'http://nohost/plone')

        self.link.remoteUrl = '${navigation_root_url}'
        self.assertEqual(view.url(), '/plone')
        self.assertEqual(view.absolute_target_url(), 'http://nohost/plone')



# -*- coding: utf-8 -*-
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.i18nmessageid import MessageFactory
from zope.interface import provider


_ = MessageFactory('plone')


@provider(IFormFieldProvider)
class ITableOfContents(model.Schema):

    model.fieldset('settings', label=_(u'Settings'),
                   fields=['table_of_contents'])

    table_of_contents = schema.Bool(
        title=_(
            u'help_enable_table_of_contents',
            default=u'Table of contents'),
        description=_(
            u'help_enable_table_of_contents_description',
            default=u'If selected, this will show a table of contents'
                    u' at the top of the page.'),
        required=False,
    )



# -*- coding: utf-8 -*-
""" Migrate Topic to DX-Collectons.

Note on Subtopics:
When a migration of Subtopics is needed, you can replace the default itemish
Collection with a folderish Collection by creating a new type folderish
type 'Collection' with the collection-behavior enabled. You can then use
the default migration to migrate Topics with Subtopics.
"""

from DateTime import DateTime
from plone.app.contenttypes.behaviors.collection import ICollection
from plone.app.contenttypes.migration.field_migrators import migrate_richtextfield  # noqa: E501
from plone.app.contenttypes.upgrades import LISTING_VIEW_MAPPING
from plone.app.querystring.interfaces import IQuerystringRegistryReader
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IMutableUUID
from Products.CMFCore.utils import getToolByName
from Products.contentmigration.inplace import InplaceCMFFolderMigrator
from Products.contentmigration.inplace import InplaceCMFItemMigrator
from Products.contentmigration.walker import CustomQueryWalker
from zope.component import getUtility
from zope.component import queryAdapter
from zope.dottedname.resolve import resolve

import logging


logger = logging.getLogger(__name__)
prefix = 'plone.app.querystring'

INVALID_OPERATION = 'Invalid operation %s for criterion: %s'


# Converters
class CriterionConverter(object):

    # Last part of the code for the dotted operation method,
    # e.g. 'string.contains'.
    operator_code = ''
    # alternative code, possibly used if the first code does not work.
    alt_operator_code = ''

    def get_query_value(self, value, index, criterion):
        # value may contain a query and some parameters, but in the
        # simple case it is simply a value.
        return value

    def get_operation(self, value, index, criterion):
        # Get dotted operation method.  This may depend on value.
        return '{0}.operation.{1}'.format(prefix, self.operator_code)

    def get_alt_operation(self, value, index, criterion):
        # Get dotted operation method.  This may depend on value.
        return '{0}.operation.{1}'.format(prefix, self.alt_operator_code)

    def is_index_known(self, registry, index):
        # Is the index registered as criterion index?
        key = '{0}.field.{1}'.format(prefix, index)
        try:
            registry.get(key)
        except KeyError:
            logger.error(
                'Index {0} is no criterion index. Registry gives '
                'KeyError: {1}'.format(index, key)
            )
            return False
        return True

    def is_index_enabled(self, registry, index):
        # Is the index enabled as criterion index?
        key = '{0}.field.{1}'.format(prefix, index)
        index_data = registry.get(key)
        if index_data.get('enabled'):
            return True
        logger.warn('Index %s is not enabled as criterion index. ', index)
        return False

    def switch_type_to_portal_type(self, value, criterion):
        # 'portal_type' is the object id of the FTI in portal_types.
        # 'Type' is the title of that object.
        # For example:
        # - portal_type 'Document' has Type 'Page'.
        # - portal_type 'Topic' has Type 'Collection (old)'.
        if isinstance(value, dict):
            values = value.get('query', [])
        else:
            values = value
        if not values:
            return value
        new_values = []
        ttool = getToolByName(criterion, 'portal_types')
        type_to_portal_type = {}
        portal_types = ttool.objectIds()
        for portal_type, Type in ttool.listTypeTitles().items():
            type_to_portal_type[Type] = portal_type
        for Type in values:
            portal_type = type_to_portal_type.get(Type)
            if not portal_type:
                if Type in portal_types:
                    portal_type = Type
                else:
                    logger.warn('Cannot switch Type %r to portal_type.', Type)
                    continue
            new_values.append(portal_type)
        if isinstance(value, dict):
            value['query'] = new_values
        else:
            value = new_values
        return value

    def is_operation_valid(self, registry, operation):
        # Check that the operation exists.
        op_info = registry.get(operation)
        if op_info is None:
            logger.error('Operation %r is not defined.', operation)
            return False
        op_function_name = op_info.get('operation')
        try:
            resolve(op_function_name)
        except ImportError:
            logger.error(
                'ImportError for operation %r: %s',
                operation,
                op_function_name,
            )
            return False
        return True

    def get_valid_operation(self, registry, index, value, criterion):
        key = '{0}.field.{1}.operations'.format(prefix, index)
        operations = registry.get(key)
        operation = self.get_operation(value, index, criterion)
        if operation not in operations:
            operation = self.get_alt_operation(value, index, criterion)
            if operation not in operations:
                return
        if self.is_operation_valid(registry, operation):
            return operation

    def add_to_formquery(self, formquery, index, operation, query_value):
        row = {'i': index,
               'o': operation}
        if query_value is not None:
            row['v'] = query_value
        formquery.append(row)

    def __call__(self, formquery, criterion, registry):
        criteria = criterion.getCriteriaItems()
        if not criteria:
            logger.warn('Ignoring empty criterion %s.', criterion)
            return
        for index, value in criteria:
            # Check if the index is known and enabled as criterion index.
            if index == 'Type':
                # Try to replace Type by portal_type
                index = 'portal_type'
                value = self.switch_type_to_portal_type(value, criterion)
            if not self.is_index_known(registry, index):
                logger.info('Index %s not known in registry.', index)
                continue
            self.is_index_enabled(registry, index)
            # TODO: what do we do when this is False?  Raise an
            # Exception?  Continue processing the index and value
            # anyway, now that a warning is logged?  Continue with the
            # next criteria item?

            # Get the operation method.
            operation = self.get_valid_operation(
                registry,
                index,
                value,
                criterion
            )
            if not operation:
                logger.error(INVALID_OPERATION % (operation, criterion))
                # TODO: raise an Exception?
                continue

            # Get the value that we will query for.
            query_value = self.get_query_value(value, index, criterion)

            # Add a row to the form query.
            self.add_to_formquery(formquery, index, operation, query_value)


class ATDateCriteriaConverter(CriterionConverter):
    """Handle date criteria.

    Note that there is also ATDateRangeCriterion, which is much
    simpler as it just has two dates.

    In our case we have these valid operations:

    ['plone.app.querystring.operation.date.lessThan',
     'plone.app.querystring.operation.date.largerThan',
     'plone.app.querystring.operation.date.between',
     'plone.app.querystring.operation.date.lessThanRelativeDate',
     'plone.app.querystring.operation.date.largerThanRelativeDate',
     'plone.app.querystring.operation.date.today',
     'plone.app.querystring.operation.date.beforeToday',
     'plone.app.querystring.operation.date.afterToday']

    This code is based on the getCriteriaItems method from
    Products/ATContentTypes/criteria/date.py.  We check the field
    values ourselves instead of translating the values back and forth.
    """

    def __call__(self, formquery, criterion, registry):  # noqa
        if criterion.value is None:
            logger.warn('Ignoring empty criterion %s.', criterion)
            return
        field = criterion.Field()
        value = criterion.Value()

        # Check if the index is known and enabled as criterion index.
        if not self.is_index_known(registry, field):
            return
        self.is_index_enabled(registry, field)

        # Negate the value for 'old' days
        if criterion.getDateRange() == '-':
            value = -value

        date = DateTime() + value

        # Get the possible operation methods.
        key = '{0}.field.{1}.operations'.format(prefix, field)
        operations = registry.get(key)

        def add_row(operation, value=None):
            if operation not in operations:
                # TODO just ignore it?
                raise ValueError(INVALID_OPERATION % (operation, criterion))
            if not self.is_operation_valid(registry, operation):
                # TODO just ignore it?
                raise ValueError(INVALID_OPERATION % (operation, criterion))
            # Add a row to the form query.
            row = {'i': field,
                   'o': operation}
            if value is not None:
                row['v'] = value
            formquery.append(row)

        operation = criterion.getOperation()
        if operation == 'within_day':
            if date.isCurrentDay():
                new_operation = '{0}.operation.date.today'.format(prefix)
                add_row(new_operation)
                return
            date_range = (date.earliestTime(), date.latestTime())
            new_operation = '{0}.operation.date.between'.format(prefix)
            add_row(new_operation, date_range)
            return
        if operation == 'more':
            if value != 0:
                new_operation = ('{0}.operation.date.'
                                 'largerThanRelativeDate'.format(prefix))
                add_row(new_operation, value)
                return
            else:
                new_operation = '{0}.operation.date.afterToday'.format(prefix)
                add_row(new_operation)
                return
        if operation == 'less':
            if value != 0:
                new_operation = ('{0}.operation.date.'
                                 'lessThanRelativeDate'.format(prefix))
                add_row(new_operation, value)
                return
            else:
                new_operation = '{0}.operation.date.beforeToday'.format(prefix)
                add_row(new_operation)
                return


class ATSimpleStringCriterionConverter(CriterionConverter):
    operator_code = 'string.contains'
    # review_state could be a string criterion, but should become a selection.
    alt_operator_code = 'selection.any'


class ATCurrentAuthorCriterionConverter(CriterionConverter):
    operator_code = 'string.currentUser'


class ATSelectionCriterionConverter(CriterionConverter):
    operator_code = 'selection.any'
    alt_operator_code = 'selection.is'

    def get_operation(self, value, index, criterion):
        # Get dotted operation method.  This may depend on value.
        if index == 'Subject':
            if value['operator'] == 'and':
                # Subject is currently the only index that supports
                # this, because for others it makes no sense.  See
                # allowed operations in registry.xml in
                # plone.app.querystring.
                suffix = 'all'
            else:
                suffix = 'any'
            return '{0}.operation.selection.{1}'.format(prefix, suffix)
        else:
            return '{0}.operation.{1}'.format(prefix, self.operator_code)

    def get_query_value(self, value, index, criterion):
        values = value['query']
        if value.get('operator') == 'and' and len(values) > 1 and \
                index != 'Subject':
            logger.warn("Cannot handle selection operator 'and'. Using 'or'. "
                        "%r", value)

        # Special handling for portal_type=Topic.
        if index == 'portal_type' and 'Topic' in values:
            values = list(values)
            values[values.index('Topic')] = 'Collection'
            values = tuple(values)
        return values


class ATListCriterionConverter(ATSelectionCriterionConverter):
    pass


class ATReferenceCriterionConverter(ATSelectionCriterionConverter):
    # Note: the new criterion is disabled by default.  Also, it
    # needs the _referenceIs function in the plone.app.querystring
    # queryparser and that function is not defined.
    operator_code = 'reference.is'


class ATPathCriterionConverter(CriterionConverter):
    operator_code = 'string.path'

    def get_query_value(self, value, index, criterion):
        raw = criterion.getRawValue()
        if not raw:
            return
        # Is this a recursive query?  Could check depth in the value
        # actually, but Recurse is the canonical way.  Also, the only
        # possible values for depth are -1 and 1.
        if not criterion.Recurse():
            for index, path in enumerate(raw):
                raw[index] = path + '::1'
        return raw

    def add_to_formquery(self, formquery, index, operation, query_value):
        if query_value is None:
            return
        for value in query_value:
            row = {'i': index,
                   'o': operation,
                   'v': value}
            formquery.append(row)


class ATBooleanCriterionConverter(CriterionConverter):

    def get_operation(self, value, index, criterion):
        # Get dotted operation method.
        # value is one of these beauties:
        # value = [1, True, '1', 'True']
        # value = [0, '', False, '0', 'False', None, (), [], {}, MV]
        if True in value:
            code = 'isTrue'
        elif False in value:
            code = 'isFalse'
        else:
            logger.warn(
                'Unknown value for boolean criterion. '
                'Falling back to True. %r',
                value,
            )
            code = 'isTrue'
        return '{0}.operation.boolean.{1}'.format(prefix, code)

    def __call__(self, formquery, criterion, registry):
        criteria = criterion.getCriteriaItems()
        if not criteria:
            return
        for index, value in criteria:
            if index == 'is_folderish':
                fieldname = 'isFolderish'
            elif index == 'is_default_page':
                fieldname = 'isDefaultPage'
            else:
                fieldname = index
            # Check if the index is known and enabled as criterion index.
            if not self.is_index_known(registry, fieldname):
                continue
            self.is_index_enabled(registry, fieldname)
            # Get the operation method.
            operation = self.get_valid_operation(
                registry, fieldname, value, criterion)
            if not operation:
                logger.error(INVALID_OPERATION % (operation, criterion))
                # TODO: raise an Exception?
                continue
            # Add a row to the form query.
            row = {'i': index,
                   'o': operation}
            formquery.append(row)


class ATDateRangeCriterionConverter(CriterionConverter):
    operator_code = 'date.between'

    def get_query_value(self, value, index, criterion):
        return value['query']


class ATPortalTypeCriterionConverter(CriterionConverter):
    operator_code = 'selection.any'

    def get_query_value(self, value, index, criterion):
        # Special handling for portal_type=Topic.
        if 'Topic' in value:
            value = list(value)
            value[value.index('Topic')] = 'Collection'
            value = tuple(value)
        return value


class ATRelativePathCriterionConverter(CriterionConverter):
    # We also have path.isWithinRelative, but its function is not defined.
    operator_code = 'string.relativePath'

    def get_query_value(self, value, index, criterion):
        if not criterion.Recurse():
            logger.warn(
                'Cannot handle non-recursive path search. '
                'Allowing recursive search. %r',
                value,
            )
        return criterion.getRelativePath()


class ATSimpleIntCriterionConverter(CriterionConverter):
    # Also available: int.lessThan, int.largerThan.
    operator_code = 'int.is'

    def get_operation(self, value, index, criterion):
        # Get dotted operation method.
        direction = value.get('range')
        if not direction:
            code = 'is'
        elif direction == 'min':
            code = 'largerThan'
        elif direction == 'max':
            code = 'lessThan'
        elif direction == 'min:max':
            logger.warn(
                'min:max direction not supported for integers. %r',
                value,
            )
            return
        else:
            logger.warn('Unknown direction for integers. %r', value)
            return
        return '{0}.operation.int.{1}'.format(prefix, code)

    def get_query_value(self, value, index, criterion):
        if isinstance(value['query'], tuple):
            logger.warn('More than one integer is not supported. %r', value)
            return
        return value['query']


class TopicMigrator(InplaceCMFItemMigrator):
    """Migrate Topics to Collections. Existing subtopics will be lost.

    The only difference to the migration below is the base-class
    (InplaceCMFItemMigrator instead of InplaceCMFFolderMigrator).
    """
    src_portal_type = 'Topic'
    src_meta_type = 'ATTopic'
    dst_portal_type = dst_meta_type = 'Collection'

    @property
    def registry(self):
        return self.kwargs['registry']

    def beforeChange_criteria(self):
        """Store the criteria of the old Topic.

        Store the info on the migrator and restore the values in the
        migrate_criteria method.
        """
        self._collection_sort_reversed = None
        self._collection_sort_on = None
        self._collection_query = None
        path = '/'.join(self.old.getPhysicalPath())
        logger.info('Migrating %s at %s', self.src_portal_type, path)
        # Get the old criteria.
        # See also Products.ATContentTypes.content.topic.buildQuery
        criteria = self.old.listCriteria()
        logger.debug(
            'Old criteria for %s: %r',
            path,
            [(crit, crit.getCriteriaItems()) for crit in criteria],
        )
        formquery = []
        for criterion in criteria:
            type_ = criterion.__class__.__name__
            if type_ == 'ATSortCriterion':
                # Sort order and direction are now stored in the Collection.
                self._collection_sort_reversed = criterion.getReversed()
                self._collection_sort_on = criterion.Field()
                logger.debug(
                    'Sort on %r, reverse: %s.',
                    self._collection_sort_on,
                    self._collection_sort_reversed,
                )
                continue

            converter = CONVERTERS.get(type_)
            if converter is None:
                msg = 'Unsupported criterion {0}'.format(type_)
                logger.error(msg)
                raise ValueError(msg)
            converter(formquery, criterion, self.registry)

        logger.debug('New query for %s: %r', path, formquery)
        self._collection_query = formquery

    def migrate_criteria(self):
        """Migrate old style to new style criteria.

        Plus handling for some special fields.
        """
        # The old Topic has boolean limitNumber and integer itemCount,
        # where the new Collection only has limit.
        adapted = ICollection(self.new)
        if self.old.getLimitNumber():
            adapted.limit = self.old.getItemCount()
        adapted.customViewFields = self.old.getCustomViewFields()

        # Get the old data stored by the beforeChange_criteria method.
        if self._collection_sort_reversed is not None:
            adapted.sort_reversed = self._collection_sort_reversed
        if self._collection_sort_on is not None:
            adapted.sort_on = self._collection_sort_on
        if self._collection_query is not None:
            adapted.query = self._collection_query

    def migrate_atctmetadata(self):
        field = self.old.getField('excludeFromNav')
        self.new.exclude_from_nav = field.get(self.old)

    def migrate_schema_fields(self):
        migrate_richtextfield(self.old, self.new, 'text', 'text')

    def migrate_at_uuid(self):
        """Migrate AT universal uid
        """
        uid = self.UID
        if uid and queryAdapter(self.new, IMutableUUID):
            IMutableUUID(self.new).set(str(uid))

    def last_migrate_layout(self):
        """Migrate the layout (view method).

        This needs to be done last, as otherwise our changes in
        migrate_criteria may get overriden by a later call to
        migrate_properties.
        """
        if self.old.getCustomView():
            # Previously, the atct_topic_view had logic for showing
            # the results in a list or in tabular form.  If
            # getCustomView is True, this means the new object should
            # use the tabular view.
            self.new.setLayout('tabular_view')
            return

        old_layout = self.old.getLayout() or getattr(self.old, 'layout', None)
        if old_layout in LISTING_VIEW_MAPPING.keys():
            self.new.setLayout(LISTING_VIEW_MAPPING[old_layout])


class FolderishTopicMigrator(InplaceCMFFolderMigrator):
    """Migrate Topics and Subtopics to folderish collections.

    The only difference to the migration above is the base-class
    (InplaceCMFFolderMigrator instead of InplaceCMFItemMigrator).
    """
    src_portal_type = 'Topic'
    src_meta_type = 'ATTopic'
    dst_portal_type = dst_meta_type = 'Collection'

    @property
    def registry(self):
        return self.kwargs['registry']

    def beforeChange_criteria(self):
        """Store the criteria of the old Topic.

        Store the info on the migrator and restore the values in the
        migrate_criteria method.
        """
        self._collection_sort_reversed = None
        self._collection_sort_on = None
        self._collection_query = None
        path = '/'.join(self.old.getPhysicalPath())
        logger.info('Migrating %s at %s', self.src_portal_type, path)
        # Get the old criteria.
        # See also Products.ATContentTypes.content.topic.buildQuery
        criteria = self.old.listCriteria()
        logger.debug('Old criteria for %s: %r', path,
                     [(crit, crit.getCriteriaItems()) for crit in criteria])
        formquery = []
        for criterion in criteria:
            type_ = criterion.__class__.__name__
            if type_ == 'ATSortCriterion':
                # Sort order and direction are now stored in the Collection.
                self._collection_sort_reversed = criterion.getReversed()
                self._collection_sort_on = criterion.Field()
                logger.debug(
                    'Sort on %r, reverse: %s.',
                    self._collection_sort_on,
                    self._collection_sort_reversed,
                )
                continue

            converter = CONVERTERS.get(type_)
            if converter is None:
                msg = 'Unsupported criterion {0}'.format(type_)
                logger.error(msg)
                raise ValueError(msg)
            converter(formquery, criterion, self.registry)

        logger.debug('New query for %s: %r', path, formquery)
        self._collection_query = formquery

    def migrate_criteria(self):
        """Migrate old style to new style criteria.

        Plus handling for some special fields.
        """
        # The old Topic has boolean limitNumber and integer itemCount,
        # where the new Collection only has limit.
        adapted = ICollection(self.new)
        if self.old.getLimitNumber():
            adapted.limit = self.old.getItemCount()
        adapted.customViewFields = self.old.getCustomViewFields()

        # Get the old data stored by the beforeChange_criteria method.
        if self._collection_sort_reversed is not None:
            adapted.sort_reversed = self._collection_sort_reversed
        if self._collection_sort_on is not None:
            adapted.sort_on = self._collection_sort_on
        if self._collection_query is not None:
            adapted.query = self._collection_query

    def migrate_atctmetadata(self):
        field = self.old.getField('excludeFromNav')
        self.new.exclude_from_nav = field.get(self.old)

    def migrate_schema_fields(self):
        migrate_richtextfield(self.old, self.new, 'text', 'text')

    def migrate_at_uuid(self):
        """Migrate AT universal uid
        """
        uid = self.UID
        if uid and queryAdapter(self.new, IMutableUUID):
            IMutableUUID(self.new).set(str(uid))

    def last_migrate_layout(self):
        """Migrate the layout (view method).

        This needs to be done last, as otherwise our changes in
        migrate_criteria may get overriden by a later call to
        migrate_properties.
        """
        if self.old.getCustomView():
            # Previously, the atct_topic_view had logic for showing
            # the results in a list or in tabular form.  If
            # getCustomView is True, this means the new object should
            # use the tabular view.
            self.new.setLayout('tabular_view')
            return

        old_layout = self.old.getLayout() or getattr(self.old, 'layout', None)
        if old_layout in LISTING_VIEW_MAPPING:
            default_page = self.old.getDefaultPage()
            self.new.setLayout(LISTING_VIEW_MAPPING[old_layout])
            if default_page:
                # any defaultPage is switched of by setLayout
                # and needs to set again
                self.new.setDefaultPage(default_page)


def migrate_topics(portal):
    """Migrate ATContentTypes Topics to plone.app.contenttypes Collections.

    This could also be used as upgrade step.
    """
    # Parse the registry to get allowed operations and pass it to the
    # migrator.
    reg = getUtility(IRegistry)
    reader = IQuerystringRegistryReader(reg)
    registry = reader.parseRegistry()
    # select migrator based on the base-class of collections
    fti = portal.portal_types['Collection']
    if fti.content_meta_type == 'Dexterity Item':
        migrator = TopicMigrator
    elif fti.content_meta_type == 'Dexterity Container':
        migrator = FolderishTopicMigrator
    walker = CustomQueryWalker(portal, migrator)(registry=registry)
    return walker


CONVERTERS = {
    # Create an instance of each converter.
    'ATBooleanCriterion': ATBooleanCriterionConverter(),
    'ATCurrentAuthorCriterion': ATCurrentAuthorCriterionConverter(),
    'ATDateCriteria': ATDateCriteriaConverter(),
    'ATDateRangeCriterion': ATDateRangeCriterionConverter(),
    'ATListCriterion': ATListCriterionConverter(),
    'ATPathCriterion': ATPathCriterionConverter(),
    'ATPortalTypeCriterion': ATPortalTypeCriterionConverter(),
    'ATReferenceCriterion': ATReferenceCriterionConverter(),
    'ATRelativePathCriterion': ATRelativePathCriterionConverter(),
    'ATSelectionCriterion': ATSelectionCriterionConverter(),
    'ATSimpleIntCriterion': ATSimpleIntCriterionConverter(),
    'ATSimpleStringCriterion': ATSimpleStringCriterionConverter(),
}



# -*- coding: utf-8 -*-
from datetime import datetime
from datetime import timedelta
from plone.app.contenttypes.content import Document
from plone.app.contenttypes.content import File
from plone.app.contenttypes.content import Folder
from plone.app.contenttypes.content import Image
from plone.app.contenttypes.content import Link
from plone.app.contenttypes.content import NewsItem
from plone.app.contenttypes.migration import dxmigration
from plone.app.contenttypes.migration import migration
from plone.app.contenttypes.migration.patches import patch_before_migration
from plone.app.contenttypes.migration.patches import undo_patch_after_migration
from plone.app.contenttypes.migration.utils import installTypeIfNeeded
from plone.app.contenttypes.migration.utils import isSchemaExtended
from plone.app.contenttypes.migration.utils import restore_references
from plone.app.contenttypes.migration.utils import store_references
from plone.app.contenttypes.migration.vocabularies import ATCT_LIST
from plone.app.contenttypes.upgrades import use_new_view_names
from plone.app.contenttypes.utils import DEFAULT_TYPES
from plone.browserlayer.interfaces import ILocalBrowserLayerType
from plone.dexterity.interfaces import IDexterityContent
from plone.dexterity.interfaces import IDexterityFTI
from plone.z3cform.layout import wrap_form
from pprint import pformat
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import get_installer
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.interfaces import HIDDEN_MODE
from zExceptions import NotFound
from zope import schema
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.interface import Interface

import logging
import pkg_resources


try:
    pkg_resources.get_distribution('collective.contentleadimage')
except pkg_resources.DistributionNotFound:
    HAS_CONTENTLEADIMAGE = False
else:
    HAS_CONTENTLEADIMAGE = True

logger = logging.getLogger(__name__)


# Average time to migrate one archetype object, in milliseconds.
# This very much depends on the size of the object and system-speed
ONE_OBJECT_MIGRATION_TIME = 500


class FixBaseClasses(BrowserView):

    def __call__(self):
        """Make sure all content objects use the proper base classes.
        Instances before version 1.0b1 had no base-class.
        To update them call @@fix_base_classes on your site-root.
        """
        out = ''
        portal_types = [
            ('Document', Document),
            ('File', File),
            ('Folder', Folder),
            ('Image', Image),
            ('Link', Link),
            ('News Item', NewsItem),
        ]
        catalog = getToolByName(self.context, 'portal_catalog')
        query = {}
        for portal_type, portal_type_class in portal_types:
            query['portal_type'] = portal_type
            results = catalog(query)
            for brain in results:
                try:
                    obj = brain.getObject()
                except (KeyError, NotFound):
                    logger.exception('Can not resolve object from brain.')
                    continue
                if IDexterityContent.providedBy(obj):
                    object_class_name = obj.__class__.__name__
                    target_class_name = portal_type_class.__name__
                    if not object_class_name == target_class_name:
                        obj.__class__ = portal_type_class
                        out += 'Make {0} use {1}\n as base class.'.format(
                            obj.Title(),
                            portal_type_class.__name__,
                        )
        return out


class MigrateFromATContentTypes(BrowserView):
    """Migrate the default-types (except event and topic).
    This view can be called directly and it will migrate all content
    provided they were not schema-extended.
    This is also called by the migration-form below with some variables.
    """

    def __call__(self,
                 migrate=False,
                 content_types='all',
                 migrate_schemaextended_content=False,
                 migrate_references=True,
                 from_form=False,
                 reindex_catalog=True,
                 patch_searchabletext=False,
                 ):

        portal = self.context

        if not from_form and migrate not in ['1', 'True', 'true', 1]:
            url1 = '{0}/@@migrate_from_atct?migrate=1'.format(
                portal.absolute_url())
            url2 = '{0}/@@atct_migrator'.format(portal.absolute_url())
            msg = u'Warning \n'
            msg += u'-------\n'
            msg += u'You are accessing "@@migrate_from_atct" directly. '
            msg += u'This will migrate all content to dexterity!\n\n'
            msg += u'Really migrate all content now: {0}\n\n'.format(url1)
            msg += u'First select what to migrate: {0}'.format(url2)
            return msg

        helpers = getMultiAdapter((portal, self.request),
                                  name='atct_migrator_helpers')
        if helpers.linguaplone_installed():
            msg = 'Warning\n'
            msg += 'Migration aborted since Products.LinguaPlone is '
            msg += 'installed. See '
            msg += 'http://github.com/plone/plone.app.contenttypes#migration '
            msg += 'for more information.'
            return msg

        stats_before = self.stats()
        starttime = datetime.now()

        self.request['plone.app.contenttypes_migration_running'] = True

        msg = 'Starting Migration\n\n'
        msg += '\n-----------------------------\n'
        msg += 'Content statictics:\n'
        msg += pformat(stats_before)
        msg += '\n-----------------------------\n'
        msg += 'Types to be migrated:\n'
        msg += pformat(content_types)
        msg += '\n-----------------------------\n'
        logger.info(msg)

        # store references on the portal
        if migrate_references:
            store_references(portal)
        catalog = portal.portal_catalog

        # Patch various things that make migration harder
        (link_integrity,
         queue_indexing,
         patch_searchabletext) = patch_before_migration(patch_searchabletext)

        not_migrated = []
        migrated_types = {}

        for (k, v) in ATCT_LIST.items():
            if content_types != 'all' and k not in content_types:
                not_migrated.append(k)
                continue
            # test if the ct is extended beyond blobimage and blobfile
            if len(isSchemaExtended(v['iface'])) > len(v['extended_fields']) \
                    and not migrate_schemaextended_content:
                not_migrated.append(k)
                continue
            query = {
                'object_provides': v['iface'].__identifier__,
                'meta_type': v['old_meta_type'],
            }
            amount_to_be_migrated = len(catalog(query))
            starttime_for_current = datetime.now()
            logger.info(
                'Start migrating {0} objects from {1} to {2}'.format(
                    amount_to_be_migrated,
                    v['old_meta_type'],
                    v['type_name'],
                )
            )
            installTypeIfNeeded(v['type_name'])

            # call the migrator
            v['migrator'](portal)

            # logging
            duration_current = datetime.now() - starttime_for_current
            duration_human = str(timedelta(seconds=duration_current.seconds))
            logger.info(
                'Finished migrating {0} objects from {1} to {2} in {3}'.format(
                    amount_to_be_migrated,
                    v['old_meta_type'],
                    v['type_name'],
                    duration_human),
            )

            # some data for the results-page
            migrated_types[k] = {}
            migrated_types[k]['amount_migrated'] = amount_to_be_migrated
            migrated_types[k]['old_meta_type'] = v['old_meta_type']
            migrated_types[k]['type_name'] = v['type_name']

        # if there are blobnewsitems we just migrate them silently.
        migration.migrate_blobnewsitems(portal)

        # make sure the view-methods on the plone site are updated
        use_new_view_names(portal, types_to_fix=['Plone Site'])

        if reindex_catalog:
            catalog.clearFindAndRebuild()

        # restore references
        if migrate_references:
            restore_references(portal)

        # Revert to the original state
        undo_patch_after_migration(
            link_integrity, queue_indexing, patch_searchabletext)

        duration = str(timedelta(seconds=(datetime.now() - starttime).seconds))
        if not_migrated:
            msg = (
                'The following types were not migrated: \n {0}'.format(
                    '\n'.join(not_migrated)
                )
            )
        else:
            msg = 'Migration successful\n\n'
        msg += '\n-----------------------------\n'
        msg += 'Migration finished in: {0}'.format(duration)
        msg += '\n-----------------------------\n'
        msg += 'Migration statictics:\n'
        msg += pformat(migrated_types)
        msg += '\n-----------------------------\n'
        msg += 'State before:\n'
        msg += pformat(stats_before)
        msg += '\n-----------------------------\n'
        msg += 'Stats after:\n'
        msg += pformat(self.stats())
        msg += '\n-----------------------------\n'
        if not from_form:
            logger.info(msg)
            return msg
        else:
            stats = {
                'duration': duration,
                'before': stats_before,
                'after': self.stats(),
                'content_types': content_types,
                'migrated_types': migrated_types,
            }
            logger.info(msg)
            return stats

    def stats(self):
        results = {}
        catalog = self.context.portal_catalog
        for brain in catalog():
            try:
                classname = brain.getObject().__class__.__name__
            except (KeyError, NotFound):
                continue
            results[classname] = results.get(classname, 0) + 1
        return results


class IATCTMigratorForm(Interface):

    content_types = schema.List(
        title=u'Existing content that can be migrated',
        description=u'Select which content types you want to migrate',
        value_type=schema.Choice(
            vocabulary='plone.app.contenttypes.migration.atctypes',
        ),
        required=False,
    )

    migrate_references = schema.Bool(
        title=u'Migrate references?',
        description=(
            u'Select this option to migrate references.'
        ),
        default=True,
        required=False,
    )

    extended_content = schema.List(
        title=(
            u'Migrate content that was extended '
            u'using archetypes.schemaextender?'
        ),
        description=(
            u'Warning: You will loose all data in the extended fields!'
        ),
        value_type=schema.Choice(
            vocabulary='plone.app.contenttypes.migration.extendedtypes',
        ),
        required=False,
    )

    reindex_catalog = schema.Bool(
        title=_(u'Rebuild the catalog after the migration.'),
        description=_(u'This operation can take a very long time.'),
        default=True,
        required=False,
    )

    patch_searchabletext = schema.Bool(
        title=_(u'Disable reindexing objects during migration?'),
        description=_(
            u'This can speed up your migration a lot if you have a lot of files with searchable text.'  # noqa: E501
        ),
        default=False,
        required=False,
    )


class ATCTMigratorForm(form.Form):
    template = ViewPageTemplateFile('atct_migrator.pt')
    results_template = ViewPageTemplateFile('atct_migrator_results.pt')

    fields = field.Fields(IATCTMigratorForm)
    fields['content_types'].widgetFactory = CheckBoxFieldWidget
    fields['extended_content'].widgetFactory = CheckBoxFieldWidget
    ignoreContext = True
    enableCSRFProtection = True

    results = None

    @button.buttonAndHandler(u'Migrate', name='migrate')
    def handle_migrate(self, action):
        data, errors = self.extractData()
        context = self.context

        if errors:
            return

        content_types = data['content_types'] or []
        content_types.extend(data['extended_content'] or [])

        migration_view = getMultiAdapter(
            (context, self.request),
            name=u'migrate_from_atct'
        )
        # store results where `render` can find them
        self.results = migration_view(
            content_types=content_types,
            migrate_schemaextended_content=True,
            migrate_references=data['migrate_references'],
            from_form=True,
            reindex_catalog=data['reindex_catalog'],
            patch_searchabletext=data['patch_searchabletext'],
        )

    def updateActions(self):
        super(ATCTMigratorForm, self).updateActions()
        self.actions['migrate'].addClass('btn-danger')

    def updateWidgets(self):
        """ Overload to set defaults and hide empty widgets. """
        form.Form.updateWidgets(self)
        # 1. Select all items in the checkboxwidget
        ct_widget = self.widgets['content_types']
        # Only override when rendering the form. When you unselect
        # everything the value will be [], not ())
        if ct_widget.value == ():
            ct_widget.value = ATCT_LIST.keys()
            # Call update again since the list-items checked-states are
            # assigned in z3c.form.browser.checkbox.CheckBoxWidget.update
            # using the widget.value.
            ct_widget.update()
        # 2. Hide empty widget
        for title, widget in self.widgets.items():
            if title not in ('content_types', 'extended_content'):
                continue
            if not len(widget.items):
                # the vocabulary is empty, we hide the widget
                widget.mode = HIDDEN_MODE

    def render(self):
        if self.results:
            return self.results_template()
        else:
            return super(ATCTMigratorForm, self).render()


class IBaseClassMigratorForm(Interface):

    changed_base_classes = schema.List(
        title=u'Changed base classes',
        description=u'Select changed base classes you want to migrate',
        value_type=schema.Choice(
            vocabulary='plone.app.contenttypes.migration.changed_base_classes',
        ),
        required=True,
    )
    migrate_to_folderish = schema.Bool(
        title=u'Migrate to folderish type?',
        description=(
            u'Select this option if you changed a type from being '
            u'itemish to being folderish but the class of the type is still '
            u'the same.'
        ),
        default=False,
    )


class BaseClassMigratorForm(form.Form):

    fields = field.Fields(IBaseClassMigratorForm)
    fields['changed_base_classes'].widgetFactory = CheckBoxFieldWidget
    ignoreContext = True
    enableCSRFProtection = True

    @button.buttonAndHandler(u'Update', name='update')
    def handle_migrate(self, action):
        data, errors = self.extractData()

        if errors:
            return

        changed_base_classes = data.get('changed_base_classes', [])
        if not changed_base_classes:
            return

        migrate_to_folderish = data.get('changed_base_classes', False)
        catalog = getToolByName(self.context, 'portal_catalog')
        migrated = []
        not_migrated = []
        for brain in catalog():
            try:
                obj = brain.getObject()
            except (KeyError, NotFound):
                continue
            old_class_name = dxmigration.get_old_class_name_string(obj)
            if old_class_name in changed_base_classes:
                if dxmigration.migrate_base_class_to_new_class(
                        obj, migrate_to_folderish=migrate_to_folderish):
                    migrated.append(obj)
                else:
                    not_migrated.append(obj)

        messages = IStatusMessage(self.request)
        info_message_template = 'There are {0} objects migrated.'
        warn_message_template = 'There are not {0} objects migrated.'
        if migrated:
            msg = info_message_template.format(len(migrated))
            messages.addStatusMessage(msg, type='info')
        if not_migrated:
            msg = warn_message_template.format(len(not_migrated))
            messages.addStatusMessage(msg, type='warn')
        self.request.response.redirect(self.request['ACTUAL_URL'])


BaseClassMigrator = wrap_form(
    BaseClassMigratorForm,
)


class ATCTMigratorHelpers(BrowserView):

    def objects_to_be_migrated(self):
        """ Return the number of AT objects in the portal """
        catalog = getToolByName(self.context, 'portal_catalog')
        query = {'meta_type': [i['old_meta_type'] for i in ATCT_LIST.values()]}
        brains = catalog(query)
        self._objects_to_be_migrated = len(brains)
        return self._objects_to_be_migrated

    def estimated_migration_time(self):
        """ Return the estimated migration time """
        total_time = self.objects_to_be_migrated() * ONE_OBJECT_MIGRATION_TIME
        hours, remainder = divmod(total_time / 1000, 3600)
        minutes, seconds = divmod(remainder, 60)
        return {
            'hours': hours,
            'minutes': minutes,
            'seconds': seconds
        }

    def linguaplone_installed(self):
        """Is Products.LinguaPlone installed?
        """
        existing = queryUtility(ILocalBrowserLayerType, name='LinguaPlone')
        return bool(existing)

    def site_has_subtopics(self):
        """Check if there are subtopics. Since Collections are itemish by
        default the migration of subtopics would fail Collections are changed
        to be folderish.
        """
        catalog = getToolByName(self.context, 'portal_catalog')
        query = {'meta_type': 'ATTopic'}
        results = []
        brains = catalog(query)
        for brain in brains:
            for item in catalog(path={'query': brain.getPath(), 'depth': 1}):
                results.append(item.getURL())
        if results:
            results = set(results)
            paths = '\n'.join(results)
            logger.info('Found {0} subtopics at: \n{1}'.format(
                len(results), paths))
            return results

    def collections_are_folderish(self):
        """Since Collections are itemish by default the migration would fail
        if there are any subtopics. As a workaround we allow to migrate to
        custom folderish Collections. The custom Collections have to fulfill
        the following criteria:
        1. The id if the type has to be Collection (not collection). You can
           change a type's id in portal_types
        2. The type has to have the collection-behavior.

        This much can even be done ttw. For the views of collections
        to work the base-class of the Collections also has to implement the
        interface `plone.app.contenttypes.interfaces.ICollection`.

        This is what such a class would look like:

            from plone.app.contenttypes.behaviors.collection import ICollection
            from plone.dexterity.content import Container
            from zope.interface import implementer

            @implementer(ICollection)
            class FolderishCollection(Container):
                pass

        You can either use a completely new fti or overwrite the default fti
        like this:

            <?xml version="1.0"?>
            <object name="Collection" meta_type="Dexterity FTI">
             <property name="klass">my.package.content.FolderishCollection
             </property>
            </object>

        """
        fti = queryUtility(IDexterityFTI, name='Collection')
        if fti and fti.content_meta_type == 'Dexterity Container':
            return True
        # test for lowercase ttw-type
        fti = queryUtility(IDexterityFTI, name='collection')
        behavior = 'plone.app.contenttypes.behaviors.collection.ICollection'
        if fti and behavior in fti.behaviors:
            logger.warn("You are trying to migrate topic to collection. "
                        "Instead you need a type 'Collection'.")

    def has_contentleadimage(self):
        return HAS_CONTENTLEADIMAGE

    def installed_types(self):
        """Which types are already Dexterity and which are not."""
        results = {}
        results['installed_with_behavior'] = []
        results['installed_without_behavior'] = []
        results['not_installed'] = []
        behavior = 'plone.app.contenttypes.behaviors.leadimage.ILeadImage'
        for type_name in DEFAULT_TYPES:
            fti = queryUtility(IDexterityFTI, name=type_name)
            if fti:
                if behavior in fti.behaviors:
                    results['installed_with_behavior'].append(type_name)
                else:
                    results['installed_without_behavior'].append(type_name)
            else:
                results['not_installed'].append(type_name)
        return results


class PACInstaller(form.Form):
    """Install p.a.c and redirect to migration-form."""

    fields = field.Fields()
    template = ViewPageTemplateFile('pac_installer.pt')
    enableCSRFProtection = True

    @property
    def pac_installable(self):
        qi = get_installer(self.context)
        pac_installed = qi.is_product_installed('plone.app.contenttypes')
        pac_installable = qi.is_product_installable('plone.app.contenttypes')
        return pac_installable and not pac_installed

    @property
    def pac_installed(self):
        qi = get_installer(self.context)
        return qi.is_product_installed('plone.app.contenttypes')

    @button.buttonAndHandler(_(u'Install'), name='install')
    def handle_install(self, action):
        """ install p.a.c
        """
        url = self.context.absolute_url()
        portal_setup = getToolByName(self.context, 'portal_setup')
        portal_setup.runAllImportStepsFromProfile(
            'profile-plone.app.contenttypes:default',
            blacklisted_steps=['typeinfo'],
        )

        # For types without any instances we want to instantly
        # replace the AT-FTI's with DX-FTI's.
        self.installTypesWithoutItems()

        url = url + '/@@atct_migrator'
        self.request.response.redirect(url)

    def installTypesWithoutItems(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        for types_name in DEFAULT_TYPES:
            if not catalog.unrestrictedSearchResults(portal_type=types_name):
                installTypeIfNeeded(types_name)

    @button.buttonAndHandler(
        _(u'label_cancel', default=u'Cancel'), name='cancel')
    def handle_cancel(self, action):
        self.request.response.redirect(self.context.absolute_url())



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

import os


version = '1.4.10'


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = \
    read('README.rst') + '\n\n' + \
    read('CHANGES.rst')

setup(name='plone.app.contenttypes',
      version=version,
      description="Default content types for Plone based on Dexterity",
      long_description=long_description,
      # Get more strings from
      # https://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
          "Development Status :: 6 - Mature",
          "Framework :: Plone",
          "Framework :: Plone :: 5.1",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
      ],
      keywords='plone content types dexterity',
      author='Plone Foundation',
      author_email='plone-developers@lists.sourceforge.net',
      url='https://github.com/plone/plone.app.contenttypes',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plone', 'plone.app'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Products.CMFPlone',
          'plone.app.contentmenu',
          'plone.app.event >= 2.0a4',
          'plone.app.dexterity >= 2.0.7',  # has a fix for INameFromFilename
          'plone.app.linkintegrity',
          'plone.app.querystring >= 1.2.2',  # custom_query support
          'plone.dexterity >= 2.2.1',  # behaviors can provide primaryfields
          'plone.app.relationfield',
          'plone.namedfile [blobs] >= 4.2.0',
          'plone.app.versioningbehavior',
          'plone.app.lockingbehavior',
          'pytz',
          'plone.app.z3cform>=1.1.0.dev0',
          'six',
          'zope.deprecation',
      ],
      extras_require={
          'test': [
              'archetypes.schemaextender',
              'lxml',
              'plone.app.robotframework [debug] > 0.9.8',  # create image and file content for Image, File and News Item if not given.  # noqa
              'plone.app.testing [robot] >= 4.2.4',  # we need ROBOT_TEST_LEVEL
              # 'plone.dexterity >= 2.3.0',  # fixes setting default values # NOT RELEASED YET. # noqa
              'Products.ATContentTypes',
              'Products.contentmigration >= 2.1.8.dev0',
              'plone.app.referenceablebehavior',
          ],
          'atrefs': [
              'plone.app.referenceablebehavior',
          ],
      },
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from plone.app.contenttypes import _
from plone.app.contenttypes.behaviors.collection import ICollection
from plone.app.contenttypes.browser.folder import FolderView
from plone.app.contenttypes.interfaces import IFolder
from plone.app.contenttypes.interfaces import IImage
from plone.memoize.view import memoize


class CollectionView(FolderView):

    @property
    def collection_behavior(self):
        return ICollection(aq_inner(self.context))

    @property
    def b_size(self):
        return getattr(self, '_b_size', self.collection_behavior.item_count)

    def results(self, **kwargs):
        """Return a content listing based result set with results from the
        collection query.

        :param **kwargs: Any keyword argument, which can be used for catalog
                         queries.
        :type  **kwargs: keyword argument

        :returns: plone.app.contentlisting based result set.
        :rtype: ``plone.app.contentlisting.interfaces.IContentListing`` based
                sequence.
        """
        # Extra filter
        contentFilter = dict(self.request.get('contentFilter', {}))
        contentFilter.update(kwargs.get('contentFilter', {}))
        kwargs.setdefault('custom_query', contentFilter)
        kwargs.setdefault('batch', True)
        kwargs.setdefault('b_size', self.b_size)
        kwargs.setdefault('b_start', self.b_start)

        results = self.collection_behavior.results(**kwargs)
        return results

    def batch(self):
        # collection is already batched.
        return self.results()

    @property
    @memoize
    def _album_results(self):
        """Get results to display an album with subalbums.
        """
        results = self.results()
        images = []
        folders = []
        for it in results:
            # TODO: potentially expensive!
            ob = it.getObject()
            if IImage.providedBy(ob):
                images.append(it)
            elif IFolder.providedBy(ob):
                folders.append(it)
        return {'images': images, 'folders': folders}

    @property
    def album_images(self):
        """Get all images within this collection.
        """
        return self._album_results['images']

    @property
    def album_folders(self):
        """Get all folders within this collection.
        """
        return self._album_results['folders']

    def tabular_fields(self):
        """Returns a list of all metadata fields from the catalog that were
           selected.
        """
        context = aq_inner(self.context)
        wrapped = ICollection(context)
        fields = wrapped.selectedViewFields()
        fields = [field[0] for field in fields]
        return fields

    def no_items_message(self):
        return _(
            'description_no_results_found',
            default=u'No results were found.'
        )



# -*- coding: utf-8 -*-
from AccessControl.SecurityInfo import ModuleSecurityInfo
from plone.app.contenttypes.utils import DEFAULT_TYPES
from Products.CMFCore.permissions import setDefaultRoles


security = ModuleSecurityInfo('plone.app.contenttypes')

TYPE_ROLES = ('Manager', 'Site Administrator', 'Owner', 'Contributor')

perms = []

for typename in DEFAULT_TYPES:
    permid = 'Add' + typename
    permname = 'plone.app.contenttypes: Add ' + typename
    security.declarePublic(permid)
    setDefaultRoles(permname, TYPE_ROLES)

AddCollection = 'plone.app.contenttypes: Add Collection'
AddDocument = 'plone.app.contenttypes: Add Document'
AddEvent = 'plone.app.contenttypes: Add Event'
AddFile = 'plone.app.contenttypes: Add File'
AddFolder = 'plone.app.contenttypes: Add Folder'
AddImage = 'plone.app.contenttypes: Add Image'
AddLink = 'plone.app.contenttypes: Add Link'
AddNewsItem = 'plone.app.contenttypes: Add News Item'



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING  # noqa
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.rfc822.interfaces import IPrimaryFieldInfo
from Products.CMFCore.utils import getToolByName

import os
import unittest


class CatalogIntegrationTest(unittest.TestCase):

    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'folder')
        self.folder = self.portal.folder
        self.folder.invokeFactory(
            'Document',
            'document'
        )
        self.folder.invokeFactory(
            'News Item',
            'news_item'
        )
        self.folder.invokeFactory(
            'Link',
            'link'
        )
        self.folder.invokeFactory(
            'Image',
            'image'
        )
        self.folder.invokeFactory(
            'File',
            'file'
        )
        self.folder.invokeFactory(
            'Collection',
            'collection'
        )
        self.folder.invokeFactory(
            'Folder',
            'folder'
        )
        self.document = self.folder.document
        self.news_item = self.folder.news_item
        self.link = self.folder.link
        self.image = self.folder.image
        self.file = self.folder.file
        self.collection = self.folder.collection
        # Note: this changes self.folder.
        self.folder = self.folder.folder
        self.catalog = getToolByName(self.portal, 'portal_catalog')

    def test_id_in_searchable_text_index(self):
        brains = self.catalog.searchResults(dict(
            SearchableText='document',
        ))
        self.assertEqual(len(brains), 1)
        self.assertEqual(
            brains[0].getPath(),
            '/plone/folder/document'
        )

    def test_title_in_searchable_text_index(self):
        self.document.title = 'My title'
        self.document.reindexObject()
        brains = self.catalog.searchResults(dict(
            SearchableText='My title',
        ))
        self.assertEqual(len(brains), 1)
        self.assertEqual(
            brains[0].getPath(),
            '/plone/folder/document'
        )

    def test_description_in_searchable_text_index(self):
        self.document.description = 'My description'
        self.document.reindexObject()
        brains = self.catalog.searchResults(dict(
            SearchableText='My description',
        ))
        self.assertEqual(len(brains), 1)
        self.assertEqual(
            brains[0].getPath(),
            '/plone/folder/document'
        )

    def test_subject_in_searchable_text_index(self):
        self.document.setSubject(['Apples', 'Oranges', ])
        self.document.reindexObject()
        brains = self.catalog.searchResults(dict(
            SearchableText='Apples',
        ))
        self.assertEqual(len(brains), 1)
        self.assertEqual(
            brains[0].getPath(),
            '/plone/folder/document'
        )

    def test_folder_fields_in_searchable_text_index(self):
        self.folder.title = 'Carpeta'
        self.folder.description = 'My description'
        self.folder.reindexObject()
        # Description
        brains = self.catalog.searchResults(dict(
            SearchableText='My description',
        ))
        self.assertEqual(len(brains), 1)
        self.assertEqual(
            brains[0].getPath(),
            '/plone/folder/folder'
        )
        # Title
        brains = self.catalog.searchResults(dict(
            SearchableText='Carpeta',
        ))
        self.assertEqual(len(brains), 1)
        self.assertEqual(
            brains[0].getPath(),
            '/plone/folder/folder'
        )

    def test_remote_url_in_searchable_text_index(self):
        self.link.remoteUrl = 'http://www.plone.org/'
        self.link.reindexObject()
        brains = self.catalog.searchResults(dict(
            SearchableText='plone',
        ))
        self.assertEqual(len(brains), 1)
        self.assertEqual(
            brains[0].getPath(),
            '/plone/folder/link'
        )

    def test_text_in_searchable_text_index(self):
        self.document.text = RichTextValue(
            u'Lorem ipsum',
            'text/plain',
            'text/html'
        )
        self.news_item.text = RichTextValue(
            u'Lorem ipsum',
            'text/plain',
            'text/html'
        )
        self.collection.text = RichTextValue(
            u'Lorem ipsum',
            'text/plain',
            'text/html'
        )
        self.document.reindexObject()
        self.news_item.reindexObject()
        self.collection.reindexObject()
        brains = self.catalog.searchResults(dict(
            SearchableText=u'Lorem ipsum',
        ))
        self.assertEqual(len(brains), 3)

        paths = [it.getPath() for it in brains]
        self.assertTrue(
            '/plone/folder/news_item' in paths
        )
        self.assertTrue(
            '/plone/folder/document' in paths
        )
        self.assertTrue(
            '/plone/folder/collection' in paths
        )

    def test_collection_text_in_searchable_text_index_after_upgrade(self):
        # At first, the text field of Collections did not end up
        # in the SearchableText index.
        # To mimic this, we reindex the object and afterwards set the text.
        self.collection.reindexObject()
        # Check that nothing is found yet.
        # This is needed to force a flush of the indexing queue.
        brains = self.catalog.searchResults(dict(
            SearchableText=u'Lorem ipsum',
        ))
        self.assertEqual(len(brains), 0)
        self.collection.text = RichTextValue(
            u'Lorem ipsum',
            'text/plain',
            'text/html'
        )
        brains = self.catalog.searchResults(dict(
            SearchableText=u'Lorem ipsum',
        ))
        self.assertEqual(len(brains), 0)
        # Now we apply the upgrade.
        from plone.app.contenttypes.upgrades import searchabletext_collections
        searchabletext_collections(self.portal.portal_setup)
        brains = self.catalog.searchResults(dict(
            SearchableText=u'Lorem ipsum',
        ))
        self.assertEqual(len(brains), 1)
        self.assertEqual(brains[0].getPath(), '/plone/folder/collection')

    def test_html_stripped_searchable_text_index(self):
        """Ensure, html tags are stripped out from the content and not indexed.
        """
        self.document.text = RichTextValue(
            u'<p>Lorem <b>ipsum</b></p>',
            mimeType='text/html',
            outputMimeType='text/html'
        )
        self.document.reindexObject()
        brains = self.catalog.searchResults(dict(
            SearchableText=u'Lorem ipsum',
        ))
        self.assertEqual(len(brains), 1)
        rid = brains[0].getRID()
        index_data = self.catalog.getIndexDataForRID(rid)
        self.assertEqual(index_data['SearchableText'].count('p'), 0)
        self.assertEqual(index_data['SearchableText'].count('b'), 0)

    def test_raw_text_searchable_text_index(self):
        """Ensure that raw text is used, instead of output.

        It makes no sense to transform raw text to the output mimetype,
        and then transform it again to plain text.
        Note that this does mean that javascript may get in the
        searchable text, but you will usually have a hard time setting it.
        """
        self.document.text = RichTextValue(
            u"""<script type="text/javascript">alert('Lorem ipsum')"""
            u"""</script>""",
            mimeType='text/html',
            outputMimeType='text/x-html-safe'
        )
        self.document.reindexObject()
        brains = self.catalog.searchResults(dict(
            SearchableText=u'Lorem ipsum',
        ))
        self.assertEqual(len(brains), 1)
        rid = brains[0].getRID()
        index_data = self.catalog.getIndexDataForRID(rid)
        self.assertEqual(index_data['SearchableText'].count('script'), 0)
        self.assertEqual(index_data['SearchableText'].count('text'), 0)

    def test_file_fulltext_in_searchable_text_index_string(self):
        from plone.namedfile.file import NamedBlobFile
        data = ('Lorem ipsum. Köln <!-- ...oder München, das ist hier die '
                'Frage. -->')
        test_file = NamedBlobFile(data=data, filename=u'string.html')

        primary_field_info = IPrimaryFieldInfo(self.file)
        primary_field_info.field.set(self.file, test_file)
        self.file.reindexObject()

        brains = self.catalog.searchResults(dict(
            SearchableText=u'Lorem ipsum'))
        self.assertEqual(len(brains), 1)

        brains = self.catalog.searchResults(dict(
            SearchableText=u'Köln'))
        self.assertEqual(len(brains), 1)

        brains = self.catalog.searchResults(dict(
            SearchableText=u'München'))
        self.assertEqual(len(brains), 0)  # hint: html comment is stripped

    def test_file_fulltext_in_searchable_text_index_unicode(self):
        from plone.namedfile.file import NamedBlobFile
        data = (u'Lorem ipsum Köln <!-- ...oder München, das ist hier die '
                u'Frage. -->')
        test_file = NamedBlobFile(data=data, filename=u'unicode.html')

        primary_field_info = IPrimaryFieldInfo(self.file)
        primary_field_info.field.set(self.file, test_file)
        self.file.reindexObject()

        brains = self.catalog.searchResults(dict(
            SearchableText=u'Lorem ipsum'))
        self.assertEqual(len(brains), 1)

        brains = self.catalog.searchResults(dict(
            SearchableText=u'Köln'))
        self.assertEqual(len(brains), 1)

        brains = self.catalog.searchResults(dict(
            SearchableText=u'München'))
        self.assertEqual(len(brains), 0)  # hint: html comment is stripped

    def test_title_in_metadata(self):
        self.document.title = 'My title'
        self.document.reindexObject()
        brains = self.catalog.searchResults(dict(
            path='/plone/folder/document',
        ))
        self.assertEqual(
            brains[0].Title,
            'My title'
        )

    def test_description_in_metadata(self):
        self.document.description = 'My description'
        self.document.reindexObject()
        brains = self.catalog.searchResults(dict(
            path='/plone/folder/document',
        ))
        self.assertEqual(
            brains[0].Description,
            'My description'
        )

    def test_get_remote_url_in_metadata(self):
        self.link.remoteUrl = 'http://www.plone.org/'
        self.link.reindexObject()
        brains = self.catalog.searchResults(dict(
            path='/plone/folder/link',
        ))
        self.assertEqual(
            brains[0].getRemoteUrl,
            'http://www.plone.org/'
        )

    def test_get_remote_url_in_metadata_variables_replaced(self):
        """Link URL must be in catalog with the variables
        ${navigation_root_url} and ${portal_url} replaced by the corresponding
        paths. Otherwise the navigation portlet will show an wrong URL for the
        link object. (See issue #110)
        """
        self.link.remoteUrl = '${navigation_root_url}/my-item'
        self.link.reindexObject()
        brains = self.catalog.searchResults(dict(
            path='/plone/folder/link',
        ))
        self.assertEqual(
            brains[0].getRemoteUrl,
            '/plone/my-item'
        )

    def test_getobjsize_image(self):
        from .test_image import dummy_image

        primary_field_info = IPrimaryFieldInfo(self.image)
        primary_field_info.field.set(self.image, dummy_image())
        self.image.reindexObject()

        brains = self.catalog.searchResults(dict(
            path='/plone/folder/image',
        ))

        # XXX: Do we still rely on getObjSize in portal_skins/plone_scripts?
        self.assertEqual(
            self.portal.getObjSize(None, primary_field_info.value.size),
            brains[0].getObjSize,
        )

    def test_getobjsize_file(self):
        from plone.namedfile.file import NamedBlobFile

        filename = os.path.join(os.path.dirname(__file__), u'image.jpg')
        test_file = NamedBlobFile(data=open(filename, 'r').read(),
                                  filename=filename)

        primary_field_info = IPrimaryFieldInfo(self.file)
        primary_field_info.field.set(self.file, test_file)
        self.file.reindexObject()

        brains = self.catalog.searchResults(dict(
            path='/plone/folder/file',
        ))

        # XXX: Do we still rely on getObjSize in portal_skins/plone_scripts?
        self.assertEqual(
            self.portal.getObjSize(None, primary_field_info.value.size),
            brains[0].getObjSize,
        )



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from Acquisition import aq_inner
from plone.app.layout.viewlets import ViewletBase
from plone.dexterity.interfaces import IDexterityFTI
from Products.CMFCore.permissions import ManagePortal

import pkg_resources


try:
    pkg_resources.get_distribution('Products.Archetypes')
except pkg_resources.DistributionNotFound:
    HAS_ARCHETYPES = False
else:
    from Products.Archetypes.interfaces.base import IBaseObject
    HAS_ARCHETYPES = True


class ATWarningViewlet(ViewletBase):

    def update(self):
        self.available = False
        if not HAS_ARCHETYPES:
            return
        self.context = aq_inner(self.context)
        replaced_types = [
            'ATFolder',
            'ATDocument',
            'ATFile',
            'ATImage',
            'ATNewsItem',
            'ATLink',
            'ATEvent',
            'ATBlobImage',
            'ATBlobFile',
            'Collection'
        ]
        if self.context.meta_type not in replaced_types:
            return
        if not IBaseObject.providedBy(self.context):
            return
        context_fti = self.context.getTypeInfo()
        if IDexterityFTI.providedBy(context_fti):
            self.available = True

    def can_migrate(self):
        sm = getSecurityManager()
        if sm.checkPermission(ManagePortal, self.context):
            return True
